@@ -140,9 +140,11 @@ protected:
 	void OnTimer(fz::timer_id);
 	void OnCommandEvent();
 
-	// General mutex for operations on the engine
 	// Todo: More fine-grained locking, a global mutex isn't nice
-	static fz::mutex mutex_;
+	static fz::mutex global_mutex_;
+	
+	// General mutex for operations on this engine
+	mutable fz::mutex mutex_;
 
 	// Used to synchronize access to the notification list
 	fz::mutex notification_mutex_{false};
@@ -156,12 +158,12 @@ protected:
 	// Indicicates if data has been received/sent and whether to send any notifications
 	static std::atomic_int m_activeStatus[2];
 
-	std::unique_ptr<CControlSocket> m_pControlSocket;
+	std::unique_ptr<CControlSocket> controlSocket_;
 
 	std::unique_ptr<CCommand> m_pCurrentCommand;
 
 	// Protect access to these three with notification_mutex_
-	std::list<CNotification*> m_NotificationList;
+	std::deque<CNotification*> m_NotificationList;
 	bool m_maySendNotificationEvent{true};
 	unsigned int m_asyncRequestCounter{};
 
@@ -197,7 +199,7 @@ protected:
 	CFileZillaEngine& parent_;
 
 	bool queue_logs_{true};
-	std::deque<CLogmsgNotification*> queued_logs_;
+	std::vector<CLogmsgNotification*> queued_logs_;
 
 	fz::thread_pool & thread_pool_;
 
