@@ -6,20 +6,29 @@
 
 #include "backend.h"
 
+#include <libfilezilla/encode.hpp>
+
 int CHttpRequestOpData::Send()
 {
-	LogMessage(MessageType::Debug_Verbose, L"CHttpRequestOpData::Send() in state %d", opState);
+	LogMessage((opState == request_send) ? MessageType::Debug_Debug : MessageType::Debug_Verbose, L"CHttpRequestOpData::Send() in state %d", opState);
 
 	switch (opState) {
 	case request_init:
 		{
+			if (request_.verb_.empty()) {
+				LogMessage(MessageType::Debug_Warning, L"No request verb");
+				return FZ_REPLY_INTERNALERROR;
+			}
 			std::string host_header = request_.uri_.host_;
 			if (request_.uri_.port_ != 0) {
 				host_header += ':';
 				host_header += fz::to_string(request_.uri_.port_);
 			}
 			request_.headers_["Host"] = host_header;
-			request_.headers_["Connection"] = "close";
+			auto pos = request_.headers_.find("Connection");
+			if (pos == request_.headers_.end()) {
+				request_.headers_["Connection"] = "close";
+			}
 			request_.headers_["User-Agent"] = fz::replaced_substrings(PACKAGE_STRING, " ", "/");
 
 			auto const cl = request_.get_header("Content-Length");
@@ -32,8 +41,8 @@ int CHttpRequestOpData::Send()
 				dataToSend_ = static_cast<uint64_t>(requestContentLength);
 			}
 
-			if ((dataToSend_ > 0) != (request_.data_request_ != 0)) {
-				LogMessage(MessageType::Debug_Warning, L"dataToSend_ > 0 does not match request_.data_request_ != 0");
+			if ((dataToSend_ > 0) && !request_.data_request_) {
+				LogMessage(MessageType::Debug_Warning, L"(dataToSend_ > 0) && !request_.data_request_");
 				return FZ_REPLY_INTERNALERROR;
 			}
 
@@ -41,16 +50,29 @@ int CHttpRequestOpData::Send()
 			response_.headers_.clear();
 
 			opState = request_wait_connect;
+
 			controlSocket_.InternalConnect(fz::to_wstring_from_utf8(request_.uri_.host_), request_.uri_.port_, request_.uri_.scheme_ == "https");
+			if (controlSocket_.is_reusing_) {
+				opState = request_send_header;
+			}
 			return FZ_REPLY_CONTINUE;
 		}
 	case request_send_header:
 		{
 			// Assemble request and headers
-			std::string command = fz::sprintf("%s %s HTTP/1.1\r\n", request_.verb_, request_.uri_.get_request());
+			std::string command = fz::sprintf("%s %s HTTP/1.1", request_.verb_, request_.uri_.get_request());
+			LogMessage(MessageType::Command, "%s", command);
+			command += "\r\n";
 
 			for (auto const& header : request_.headers_) {
-				command += fz::sprintf("%s: %s\r\n", header.first, header.second);
+				std::string line = fz::sprintf("%s: %s", header.first, header.second);
+				if (header.first == "Authorization") {
+					LogMessage(MessageType::Command, "%s: %s", header.first, std::string(header.second.size(), '*'));
+				}
+				else {
+					LogMessage(MessageType::Command, "%s", line);
+				}
+				command += line + "\r\n";
 			}
 
 			command += "\r\n";
@@ -62,21 +84,23 @@ int CHttpRequestOpData::Send()
 				opState = request_read;
 			}
 
-			return controlSocket_.Send(command.c_str(), command.size());
+			auto result = controlSocket_.Send(command.c_str(), command.size());
+			if (result == FZ_REPLY_WOULDBLOCK && opState == request_send && !controlSocket_.sendBuffer_) {
+				result = FZ_REPLY_CONTINUE;
+			}
+			return result;
 		}
 	case request_send:
 		{
 			int const chunkSize = 65536;
 
-			while (dataToSend_) {
-				controlSocket_.SendBufferReserve(chunkSize);
-
-				if (!controlSocket_.sendBufferSize_) {
+			while (dataToSend_ || controlSocket_.sendBuffer_) {
+				if (!controlSocket_.sendBuffer_) {
 					unsigned int len = chunkSize;
 					if (chunkSize > dataToSend_) {
 						len = static_cast<unsigned int>(dataToSend_);
 					}
-					int res = request_.data_request_(controlSocket_.sendBuffer_ + controlSocket_.sendBufferSize_, len);
+					int res = request_.data_request_(controlSocket_.sendBuffer_.get(len), len);
 					if (res != FZ_REPLY_CONTINUE) {
 						return res;
 					}
@@ -85,24 +109,25 @@ int CHttpRequestOpData::Send()
 						return FZ_REPLY_INTERNALERROR;
 					}
 
-					controlSocket_.sendBufferSize_ = len;
+					controlSocket_.sendBuffer_.add(len);
+					dataToSend_ -= len;
 				}
 
 				int error;
-				int written = controlSocket_.m_pBackend->Write(controlSocket_.sendBuffer_ + controlSocket_.sendBufferPos_, controlSocket_.sendBufferSize_ - controlSocket_.sendBufferPos_, error);
+				int written = controlSocket_.m_pBackend->Write(controlSocket_.sendBuffer_.get(), controlSocket_.sendBuffer_.size(), error);
 				if (written < 0) {
 					if (error != EAGAIN) {
-						LogMessage(MessageType::Error, _("Could not write to socket: %s"), CSocket::GetErrorDescription(error));
+						LogMessage(MessageType::Error, _("Could not write to socket: %s"), fz::socket::error_description(error));
 						LogMessage(MessageType::Error, _("Disconnected from server"));
 						return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 					}
-					written = 0;
+					return FZ_REPLY_WOULDBLOCK;
 				}
 
-				dataToSend_ -= written;
-				if (static_cast<unsigned int>(written) != controlSocket_.sendBufferSize_ - controlSocket_.sendBufferPos_) {
-					controlSocket_.sendBufferPos_ += written;
-					controlSocket_.sendBufferSize_ -= written;
+				if (written) {
+					controlSocket_.SetActive(CFileZillaEngine::send);
+
+					controlSocket_.sendBuffer_.consume(static_cast<size_t>(written));
 				}
 			}
 			opState = request_read;
@@ -134,38 +159,29 @@ int CHttpRequestOpData::SubcommandResult(int, COpData const&)
 
 int CHttpRequestOpData::OnReceive()
 {
-	while (true) {
-		const CSocket::SocketState state = controlSocket_.m_pSocket->GetState();
-		if (state != CSocket::connected && state != CSocket::closing) {
+	while (controlSocket_.socket_) {
+		const fz::socket::socket_state state = controlSocket_.socket_->get_state();
+		if (state != fz::socket::connected && state != fz::socket::closing) {
 			return FZ_REPLY_WOULDBLOCK;
 		}
 
-		if (!recv_buffer_) {
-			recv_buffer_ = std::make_unique<unsigned char[]>(m_recvBufferLen);
-			m_recvBufferPos = 0;
-		}
-
-		unsigned int len = m_recvBufferLen - m_recvBufferPos;
-		if (!len) {
-			LogMessage(MessageType::Debug_Warning, L"Read data isn't being consumed.");
-			return FZ_REPLY_INTERNALERROR;
-		}
-
 		int error;
-		int read = controlSocket_.m_pBackend->Read(recv_buffer_.get() + m_recvBufferPos, len, error);
-		if (read == -1) {
+		size_t const recv_size = 1024 * 16;
+		int read = controlSocket_.m_pBackend->Read(recv_buffer_.get(recv_size), recv_size, error);
+		if (read <= -1) {
 			if (error != EAGAIN) {
+				LogMessage(MessageType::Error, _("Could not read from socket: %s"), fz::socket::error_description(error));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 			return FZ_REPLY_WOULDBLOCK;
 		}
+		recv_buffer_.add(static_cast<size_t>(read));
 
 		controlSocket_.SetActive(CFileZillaEngine::recv);
 
-		m_recvBufferPos += read;
-
 		if (!got_header_) {
 			if (!read) {
+				LogMessage(MessageType::Error, _("Connection closed by server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 
@@ -176,6 +192,7 @@ int CHttpRequestOpData::OnReceive()
 		}
 		else if (transfer_encoding_ == chunked) {
 			if (!read) {
+				LogMessage(MessageType::Error, _("Connection closed by server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 			int res = ParseChunkedData();
@@ -185,18 +202,25 @@ int CHttpRequestOpData::OnReceive()
 		}
 		else {
 			if (!read) {
-				assert(!m_recvBufferPos);
-				return FZ_REPLY_OK;
+				assert(recv_buffer_.empty());
+
+				if (responseContentLength_ != -1 && receivedData_ != responseContentLength_) {
+					LogMessage(MessageType::Error, _("Connection closed by server"));
+					return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+				}
+				else {
+					return FZ_REPLY_OK;
+				}
 			}
 			else {
-				int res = ProcessData(recv_buffer_.get(), m_recvBufferPos);
+				int res = ProcessData(recv_buffer_.get(), recv_buffer_.size());
+				recv_buffer_.clear();
 				if (res != FZ_REPLY_CONTINUE) {
 					return res;
-				}		
-				m_recvBufferPos = 0;
+				}
 			}
 		}
-	} while (controlSocket_.m_pSocket);
+	}
 
 	return FZ_REPLY_WOULDBLOCK;
 }
@@ -207,12 +231,11 @@ int CHttpRequestOpData::ParseHeader()
 
 	// Parse the HTTP header.
 	// We do just the neccessary parsing and silently ignore most header fields
-	// Redirects are supported though if the server sends the Location field.
-
+	// The calling operation is responsible for things like redirect parsing.
 	for (;;) {
 		// Find line ending
-		unsigned int i = 0;
-		for (i = 0; (i + 1) < m_recvBufferPos; ++i) {
+		size_t i = 0;
+		for (i = 0; (i + 1) < recv_buffer_.size(); ++i) {
 			if (recv_buffer_[i] == '\r') {
 				if (recv_buffer_[i + 1] != '\n') {
 					LogMessage(MessageType::Error, _("Malformed response header: %s"), _("Server not sending proper line endings"));
@@ -220,27 +243,30 @@ int CHttpRequestOpData::ParseHeader()
 				}
 				break;
 			}
+			if (!recv_buffer_[i]) {
+				LogMessage(MessageType::Error, _("Malformed response header: %s"), _("Null character in line"));
+				return FZ_REPLY_ERROR;
+			}
 		}
-		if ((i + 1) >= m_recvBufferPos) {
-			if (m_recvBufferPos == m_recvBufferLen) {
-				// We don't support header lines larger than 4096
+		if ((i + 1) >= recv_buffer_.size()) {
+			size_t const max_line_size = 8192;
+			if (recv_buffer_.size() >= max_line_size) {
 				LogMessage(MessageType::Error, _("Too long header line"));
 				return FZ_REPLY_ERROR;
 			}
 			return FZ_REPLY_WOULDBLOCK;
 		}
 
-		recv_buffer_[i] = 0;
-		std::wstring wline = fz::to_wstring_from_utf8(reinterpret_cast<char*>(recv_buffer_.get()));
+		std::wstring wline = fz::to_wstring_from_utf8(reinterpret_cast<char const*>(recv_buffer_.get()), i);
 		if (wline.empty()) {
-			wline = fz::to_wstring(reinterpret_cast<char*>(recv_buffer_.get()));
+			wline = fz::to_wstring(std::string(recv_buffer_.get(), recv_buffer_.get() + i));
 		}
 		if (!wline.empty()) {
 			controlSocket_.LogMessageRaw(MessageType::Response, wline);
 		}
 
 		if (!response_.code_) {
-			if (m_recvBufferPos < 16 || memcmp(recv_buffer_.get(), "HTTP/1.", 7)) {
+			if (recv_buffer_.size() < 15 || memcmp(recv_buffer_.get(), "HTTP/1.", 7)) {
 				// Invalid HTTP Status-Line
 				LogMessage(MessageType::Error, _("Invalid HTTP Response"));
 				return FZ_REPLY_ERROR;
@@ -259,8 +285,7 @@ int CHttpRequestOpData::ParseHeader()
 		}
 		else {
 			if (!i) {
-				memmove(recv_buffer_.get(), recv_buffer_.get() + 2, m_recvBufferPos - 2);
-				m_recvBufferPos -= 2;
+				recv_buffer_.consume(2);
 
 				// End of header
 				int res = ProcessCompleteHeader();
@@ -268,10 +293,6 @@ int CHttpRequestOpData::ParseHeader()
 					return res;
 				}
 
-				if (!m_recvBufferPos) {
-					return FZ_REPLY_WOULDBLOCK;
-				}
-
 				if (!got_header_) {
 					// In case we got 100 Continue
 					continue;
@@ -281,30 +302,52 @@ int CHttpRequestOpData::ParseHeader()
 					return ParseChunkedData();
 				}
 				else {
-					int res = ProcessData(recv_buffer_.get(), m_recvBufferPos);
+					if (recv_buffer_.empty()) {
+						if (!responseContentLength_) {
+							opState = request_done;
+							return FZ_REPLY_OK;
+						}
+						else {
+							return FZ_REPLY_WOULDBLOCK;
+						}
+					}
+
+					int res = ProcessData(recv_buffer_.get(), recv_buffer_.size());
+					recv_buffer_.clear();
 					if (res != FZ_REPLY_CONTINUE) {
 						return res;
 					}
-					m_recvBufferPos = 0;
 					return FZ_REPLY_WOULDBLOCK;
 				}
 			}
 
 			std::string line(recv_buffer_.get(), recv_buffer_.get() + i);
 
-			auto pos = line.find(": ");
-			if (pos == std::string::npos || !pos) {
+			auto delim_pos = line.find(':');
+			if (delim_pos == std::string::npos || !delim_pos) {
 				LogMessage(MessageType::Error, _("Malformed response header: %s"), _("Invalid line"));
 				return FZ_REPLY_ERROR;
 			}
 
-			response_.headers_[line.substr(0, pos)] = line.substr(pos + 2);
+			std::string value;
+			auto value_start = line.find_first_not_of(" \t", delim_pos + 1);
+			if (value_start != std::string::npos) {
+				int value_stop = line.find_last_not_of(" \t"); // Cannot fail
+				value = line.substr(value_start, value_stop - value_start + 1);
+			}
+
+			auto & header = response_.headers_[line.substr(0, delim_pos)];
+			if (header.empty()) {
+				header = value;
+			}
+			else if (!value.empty()) {
+				header += ", " + value;
+			}
 		}
 
-		memmove(recv_buffer_.get(), recv_buffer_.get() + i + 2, m_recvBufferPos - i - 2);
-		m_recvBufferPos -= i + 2;
+		recv_buffer_.consume(i + 2);
 
-		if (!m_recvBufferPos) {
+		if (recv_buffer_.empty()) {
 			break;
 		}
 	}
@@ -351,64 +394,71 @@ int CHttpRequestOpData::ProcessCompleteHeader()
 		res = response_.on_header_();
 	}
 
+	if (res == FZ_REPLY_CONTINUE) {
+		if (request_.verb_ == "HEAD" || response_.code_prohobits_body()) {
+			if (!recv_buffer_.empty()) {
+				LogMessage(MessageType::Error, _("Malformed response: Server sent response body with a request verb or response code that disallows a response body."));
+				return FZ_REPLY_ERROR;
+			}
+			else {
+				opState = request_done;
+				return FZ_REPLY_OK;
+			}
+		}
+	}
+
 	return res;
 }
 
 int CHttpRequestOpData::ParseChunkedData()
 {
-	unsigned char* p = recv_buffer_.get();
-	unsigned int len = m_recvBufferPos;
-
-	for (;;) {
+	while (!recv_buffer_.empty()) {
 		if (chunk_data_.size != 0) {
-			unsigned int dataLen = len;
-			if (chunk_data_.size < len) {
-				dataLen = static_cast<unsigned int>(chunk_data_.size);
+			size_t dataLen = recv_buffer_.size();
+			if (chunk_data_.size < recv_buffer_.size()) {
+				dataLen = chunk_data_.size;
 			}
-			int res = ProcessData(p, dataLen);
+			int res = ProcessData(recv_buffer_.get(), dataLen);
 			if (res != FZ_REPLY_CONTINUE) {
 				return res;
 			}
-
+			recv_buffer_.consume(dataLen);
 			chunk_data_.size -= dataLen;
-			p += dataLen;
-			len -= dataLen;
 
 			if (chunk_data_.size == 0) {
 				chunk_data_.terminateChunk = true;
 			}
-
-			if (!len) {
-				break;
-			}
 		}
 
 		// Find line ending
-		unsigned int i = 0;
-		for (i = 0; (i + 1) < len; ++i) {
-			if (p[i] == '\r') {
-				if (p[i + 1] != '\n') {
+		size_t i = 0;
+		for (i = 0; (i + 1) < recv_buffer_.size(); ++i) {
+			if (recv_buffer_[i] == '\r') {
+				if (recv_buffer_[i + 1] != '\n') {
 					LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Wrong line endings"));
 					return FZ_REPLY_ERROR;
 				}
 				break;
 			}
+			if (!recv_buffer_[i]) {
+				LogMessage(MessageType::Error, _("Malformed response header: %s"), _("Null character in line"));
+				return FZ_REPLY_ERROR;
+			}
 		}
-		if ((i + 1) >= len) {
-			if (len == m_recvBufferLen) {
-				// We don't support lines larger than 4096
+		if ((i + 1) >= recv_buffer_.size()) {
+			size_t const max_line_size = 8192;
+			if (recv_buffer_.size() >= max_line_size) {
 				LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Line length exceeded"));
 				return FZ_REPLY_ERROR;
 			}
 			break;
 		}
 
-		p[i] = 0;
-
 		if (chunk_data_.terminateChunk) {
 			if (i) {
 				// The chunk data has to end with CRLF. If i is nonzero,
 				// it didn't end with just CRLF.
+				LogMessage(MessageType::Debug_Debug, L"%u characters preceeding line-ending with value %s", i, fz::hex_encode<std::string>(std::string(recv_buffer_.get(), recv_buffer_.get() + recv_buffer_.size())));
 				LogMessage(MessageType::Error, _("Malformed chunk data: %s"), _("Chunk data improperly terminated"));
 				return FZ_REPLY_ERROR;
 			}
@@ -417,6 +467,10 @@ int CHttpRequestOpData::ParseChunkedData()
 		else if (chunk_data_.getTrailer) {
 			if (!i) {
 				// We're done
+				opState = request_done;
+
+				recv_buffer_.consume(2);
+
 				return FZ_REPLY_OK;
 			}
 
@@ -424,7 +478,8 @@ int CHttpRequestOpData::ParseChunkedData()
 		}
 		else {
 			// Read chunk size
-			for (unsigned char* q = p; *q && *q != ';' && *q != ' '; ++q) {
+			unsigned char const* end = recv_buffer_.get() + i;
+			for (unsigned char* q = recv_buffer_.get(); q != end && *q != ';' && *q != ' '; ++q) {
 				chunk_data_.size *= 16;
 				if (*q >= '0' && *q <= '9') {
 					chunk_data_.size += *q - '0';
@@ -446,17 +501,7 @@ int CHttpRequestOpData::ParseChunkedData()
 			}
 		}
 
-		p += i + 2;
-		len -= i + 2;
-
-		if (!len) {
-			break;
-		}
-	}
-
-	if (p != recv_buffer_.get()) {
-		memmove(recv_buffer_.get(), p, len);
-		m_recvBufferPos = len;
+		recv_buffer_.consume(i + 2);
 	}
 
 	return FZ_REPLY_WOULDBLOCK;
@@ -475,6 +520,11 @@ int CHttpRequestOpData::ProcessData(unsigned char* data, unsigned int len)
 		res = response_.on_data_(data, len);
 	}
 
+	if (res == FZ_REPLY_CONTINUE && receivedData_ == responseContentLength_) {
+		opState = request_done;
+		res = FZ_REPLY_OK;
+	}
+
 	return res;
 }
 
@@ -507,3 +557,30 @@ int CHttpRequestOpData::OnClose()
 
 	return FZ_REPLY_OK;
 }
+
+int CHttpRequestOpData::Reset(int result)
+{
+	LogMessage(MessageType::Debug_Verbose, L"CHttpRequestOpData::Reset(%d) in state %d", result, opState);
+
+	if (result != FZ_REPLY_OK) {
+		controlSocket_.ResetSocket();
+	}
+	else if (opState != request_done) {
+		controlSocket_.ResetSocket();
+	}
+	else if (response_.get_header("Connection") == "close" || request_.get_header("Connection") == "close") {
+		if (request_.get_header("Connection") != "close") {
+			LogMessage(MessageType::Debug_Verbose, L"Server closed connection despite request to use keep-alive");
+		}
+		controlSocket_.ResetSocket();
+	}
+	else if (!recv_buffer_.empty()) {
+		LogMessage(MessageType::Debug_Verbose, L"Closing connection, the receive buffer isn't empty but at %d", recv_buffer_.size());
+		controlSocket_.ResetSocket();
+	}
+	else {
+		controlSocket_.send_event<fz::socket_event>(controlSocket_.m_pBackend, fz::socket_event_flag::read, 0);
+	}
+
+	return result;
+}
