@@ -37,8 +37,11 @@ CTransferStatus const& CTransferStatusNotification::GetStatus() const
 	return status_;
 }
 
-CHostKeyNotification::CHostKeyNotification(std::wstring const& host, int port, std::wstring const& fingerprint, bool changed)
-	: m_host(host), m_port(port), m_fingerprint(fingerprint), m_changed(changed)
+CHostKeyNotification::CHostKeyNotification(std::wstring const& host, int port, CSftpEncryptionDetails const& details, bool changed)
+	: CSftpEncryptionDetails(details)
+	, m_host(host)
+	, m_port(port)
+	, m_changed(changed)
 {
 }
 
@@ -57,11 +60,6 @@ int CHostKeyNotification::GetPort() const
 	return m_port;
 }
 
-std::wstring CHostKeyNotification::GetFingerprint() const
-{
-	return m_fingerprint;
-}
-
 CDataNotification::CDataNotification(char* pData, int len)
 	: m_pData(pData), m_len(len)
 {
@@ -76,7 +74,7 @@ char* CDataNotification::Detach(int& len)
 {
 	len = m_len;
 	char* pData = m_pData;
-	m_pData = 0;
+	m_pData = nullptr;
 	return pData;
 }
 
@@ -138,7 +136,8 @@ CCertificateNotification::CCertificateNotification(std::wstring const& host, uns
 		std::wstring const& sessionCipher,
 		std::wstring const& sessionMac,
 		int algorithmWarnings,
-		std::vector<CCertificate> && certificates)
+		std::vector<CCertificate> && certificates,
+		bool hostnameMismatch)
 	: m_host(host)
 	, m_port(port)
 	, m_protocol(protocol)
@@ -147,5 +146,6 @@ CCertificateNotification::CCertificateNotification(std::wstring const& host, uns
 	, m_sessionMac(sessionMac)
 	, m_algorithmWarnings(algorithmWarnings)
 	, m_certificates(certificates)
+	, hostnameMismatch_(hostnameMismatch)
 {
 }
