@@ -20,13 +20,13 @@ int CSftpListOpData::Send()
 			LogMessage(MessageType::Debug_Warning, L"currenServer_ is empty");
 			return FZ_REPLY_INTERNALERROR;
 		}
-		
+
 		if (path_.GetType() == DEFAULT) {
 			path_.SetType(currentServer_.GetType());
 		}
 		refresh_ = (flags_ & LIST_FLAG_REFRESH) != 0;
 		fallback_to_current_ = !path_.empty() && (flags_ & LIST_FLAG_FALLBACK_CURRENT) != 0;
-		
+
 		controlSocket_.ChangeDir(path_, subDir_, (flags_ & LIST_FLAG_LINK) != 0);
 		opState = list_waitcwd;
 		return FZ_REPLY_CONTINUE;
@@ -85,11 +85,11 @@ int CSftpListOpData::ParseResponse()
 		return FZ_REPLY_OK;
 	}
 
-	LogMessage(MessageType::Debug_Warning, L"ListParseResponse called at inproper time: %d", opState);
+	LogMessage(MessageType::Debug_Warning, L"ListParseResponse called at improper time: %d", opState);
 	return FZ_REPLY_INTERNALERROR;
 }
 
-int CSftpListOpData::SubcommandResult(int prevResult, COpData const& previousOperation)
+int CSftpListOpData::SubcommandResult(int prevResult, COpData const&)
 {
 	LogMessage(MessageType::Debug_Verbose, L"CSftpListOpData::SubcommandResult() in state %d", opState);
 
@@ -121,7 +121,7 @@ int CSftpListOpData::ParseEntry(std::wstring && entry, std::wstring const& stime
 {
 	if (opState != list_list) {
 		controlSocket_.LogMessageRaw(MessageType::RawList, entry);
-		LogMessage(MessageType::Debug_Warning, L"ListParseResponse called at inproper time: %d", opState);
+		LogMessage(MessageType::Debug_Warning, L"ListParseResponse called at improper time: %d", opState);
 		return FZ_REPLY_INTERNALERROR;
 	}
 
@@ -133,7 +133,7 @@ int CSftpListOpData::ParseEntry(std::wstring && entry, std::wstring const& stime
 
 	fz::datetime time;
 	if (!stime.empty()) {
-		int64_t t = std::wcstoll(stime.c_str(), 0, 10);
+		int64_t t = std::wcstoll(stime.c_str(), nullptr, 10);
 		if (t > 0) {
 			time = fz::datetime(static_cast<time_t>(t), fz::datetime::seconds);
 		}
