@@ -98,7 +98,7 @@ struct cert_list_holder final
 struct datum_holder final : gnutls_datum_t
 {
 	datum_holder() {
-		data = 0;
+		data = nullptr;
 	}
 
 	~datum_holder() {
@@ -113,7 +113,7 @@ struct datum_holder final : gnutls_datum_t
 void clone_cert(gnutls_x509_crt_t in, gnutls_x509_crt_t &out)
 {
 	gnutls_x509_crt_deinit(out);
-	out = 0;
+	out = nullptr;
 
 	if (in) {
 		datum_holder der;
@@ -121,20 +121,20 @@ void clone_cert(gnutls_x509_crt_t in, gnutls_x509_crt_t &out)
 			gnutls_x509_crt_init(&out);
 			if (gnutls_x509_crt_import(out, &der, GNUTLS_X509_FMT_DER) != GNUTLS_E_SUCCESS) {
 				gnutls_x509_crt_deinit(out);
-				out = 0;
+				out = nullptr;
 			}
 		}
 	}
 }
 }
 
-CTlsSocketImpl::CTlsSocketImpl(CTlsSocket& tlsSocket, CSocket& socket, CControlSocket* pOwner)
+CTlsSocketImpl::CTlsSocketImpl(CTlsSocket& tlsSocket, fz::socket& socket, CControlSocket* pOwner)
 	: tlsSocket_(tlsSocket)
 	, m_pOwner(pOwner)
 	, m_socket(socket)
 	, socketBackend_(std::make_unique<CSocketBackend>(static_cast<fz::event_handler*>(&tlsSocket_), m_socket, m_pOwner->GetEngine().GetRateLimiter()))
 {
-	m_implicitTrustedCert.data = 0;
+	m_implicitTrustedCert.data = nullptr;
 	m_implicitTrustedCert.size = 0;
 }
 
@@ -189,7 +189,7 @@ bool CTlsSocketImpl::SetClientCertificate(fz::native_string const& keyfile, fz::
 	}
 
 	int res = gnutls_certificate_set_x509_key_file2(m_certCredentials, fz::to_string(certs).c_str(),
-		fz::to_string(keyfile).c_str(), GNUTLS_X509_FMT_PEM, password.empty() ? 0 : fz::to_utf8(password).c_str(), 0);
+		fz::to_string(keyfile).c_str(), GNUTLS_X509_FMT_PEM, password.empty() ? nullptr : fz::to_utf8(password).c_str(), 0);
 	if (res < 0) {
 		LogError(res, L"gnutls_certificate_set_x509_key_file2");
 		Uninit();
@@ -222,7 +222,7 @@ bool CTlsSocketImpl::InitSession()
 	// session is cached or not.
 	gnutls_db_set_cache_expiration(m_session, 100000000);
 
-	res = gnutls_priority_set_direct(m_session, ciphers, 0);
+	res = gnutls_priority_set_direct(m_session, ciphers, nullptr);
 	if (res) {
 		LogError(res, L"gnutls_priority_set_direct");
 		Uninit();
@@ -247,7 +247,7 @@ void CTlsSocketImpl::Uninit()
 
 	if (m_certCredentials) {
 		gnutls_certificate_free_credentials(m_certCredentials);
-		m_certCredentials = 0;
+		m_certCredentials = nullptr;
 	}
 
 	if (m_initialized) {
@@ -257,16 +257,15 @@ void CTlsSocketImpl::Uninit()
 
 	m_tlsState = CTlsSocket::TlsState::noconn;
 
-	delete [] m_peekData;
-	m_peekData = 0;
-	m_peekDataLen = 0;
+	peekBuffer_.clear();
 
 	delete [] m_implicitTrustedCert.data;
-	m_implicitTrustedCert.data = 0;
+	m_implicitTrustedCert.data = nullptr;
 
 #if TLSDEBUG
-	if (pLoggingControlSocket == m_pOwner)
-		pLoggingControlSocket = 0;
+	if (pLoggingControlSocket == m_pOwner) {
+		pLoggingControlSocket = nullptr;
+	}
 #endif
 }
 
@@ -275,7 +274,7 @@ void CTlsSocketImpl::UninitSession()
 {
 	if (m_session) {
 		gnutls_deinit(m_session);
-		m_session = 0;
+		m_session = nullptr;
 	}
 }
 
@@ -287,18 +286,18 @@ void CTlsSocketImpl::LogError(int code, std::wstring const& function, MessageTyp
 	}
 	else if (code == GNUTLS_E_PULL_ERROR) {
 		if (function.empty()) {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not read from socket: %s", CSocket::GetErrorDescription(m_socket_error));
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not read from socket: %s", fz::socket::error_description(m_socket_error));
 		}
 		else {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not read from socket in %s: %s", function, CSocket::GetErrorDescription(m_socket_error));
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not read from socket in %s: %s", function, fz::socket::error_description(m_socket_error));
 		}
 	}
 	else if (code == GNUTLS_E_PUSH_ERROR) {
 		if (function.empty()) {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not write to socket: %s", CSocket::GetErrorDescription(m_socket_error));
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not write to socket: %s", fz::socket::error_description(m_socket_error));
 		}
 		else {
-			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not write to socket in %s: %s", function, CSocket::GetErrorDescription(m_socket_error));
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"GnuTLS could not write to socket in %s: %s", function, fz::socket::error_description(m_socket_error));
 		}
 	}
 	else {
@@ -396,7 +395,7 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 		m_canReadFromSocket = false;
 		if (error == EAGAIN) {
 			if (m_canCheckCloseSocket && !socketBackend_->IsWaiting(CRateLimiter::inbound)) {
-				tlsSocket_.send_event<CSocketEvent>(socketBackend_.get(), SocketEventType::close, 0);
+				tlsSocket_.send_event<fz::socket_event>(socketBackend_.get(), fz::socket_event_flag::close, 0);
 			}
 		}
 		else {
@@ -410,7 +409,7 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 	}
 
 	if (m_canCheckCloseSocket) {
-		tlsSocket_.send_event<CSocketEvent>(socketBackend_.get(), SocketEventType::close, 0);
+		tlsSocket_.send_event<fz::socket_event>(socketBackend_.get(), fz::socket_event_flag::close, 0);
 	}
 
 	if (!read) {
@@ -426,10 +425,10 @@ ssize_t CTlsSocketImpl::PullFunction(void* data, size_t len)
 
 void CTlsSocketImpl::operator()(fz::event_base const& ev)
 {
-	fz::dispatch<CSocketEvent>(ev, this, &CTlsSocketImpl::OnSocketEvent);
+	fz::dispatch<fz::socket_event>(ev, this, &CTlsSocketImpl::OnSocketEvent);
 }
 
-void CTlsSocketImpl::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
+void CTlsSocketImpl::OnSocketEvent(fz::socket_event_source*, fz::socket_event_flag t, int error)
 {
 	if (!m_session) {
 		return;
@@ -437,13 +436,13 @@ void CTlsSocketImpl::OnSocketEvent(CSocketEventSource*, SocketEventType t, int e
 
 	switch (t)
 	{
-	case SocketEventType::read:
+	case fz::socket_event_flag::read:
 		OnRead();
 		break;
-	case SocketEventType::write:
+	case fz::socket_event_flag::write:
 		OnSend();
 		break;
-	case SocketEventType::close:
+	case fz::socket_event_flag::close:
 		{
 			m_canCheckCloseSocket = true;
 			char tmp[100];
@@ -465,7 +464,7 @@ void CTlsSocketImpl::OnSocketEvent(CSocketEventSource*, SocketEventType t, int e
 
 			m_pOwner->LogMessage(MessageType::Debug_Info, L"CTlsSocketImpl::OnSocketEvent(): close event received");
 
-			tlsSocket_.m_pEvtHandler->send_event<CSocketEvent>(&tlsSocket_, SocketEventType::close, 0);
+			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::close, 0);
 		}
 		break;
 	default:
@@ -568,8 +567,6 @@ int CTlsSocketImpl::Handshake(const CTlsSocketImpl* pPrimarySocket, bool try_res
 
 	m_tlsState = CTlsSocket::TlsState::handshake;
 
-	fz::native_string hostname;
-
 	if (pPrimarySocket) {
 		if (!pPrimarySocket->m_session) {
 			m_pOwner->LogMessage(MessageType::Debug_Warning, L"Primary socket has no session");
@@ -592,14 +589,28 @@ int CTlsSocketImpl::Handshake(const CTlsSocketImpl* pPrimarySocket, bool try_res
 			}
 		}
 
-		hostname = pPrimarySocket->m_socket.GetPeerHost();
+		hostname_ = pPrimarySocket->m_socket.peer_host();
+
+		int port, tmp;
+		port = m_socket.remote_port(tmp);
+		if (port <= 0) {
+			return FZ_REPLY_ERROR;
+		}
+		port_ = port;
 	}
 	else {
-		hostname = m_socket.GetPeerHost();
+		hostname_ = m_socket.peer_host();
+
+		int port, tmp;
+		port = m_socket.remote_port(tmp);
+		if (port <= 0) {
+			return FZ_REPLY_ERROR;
+		}
+		port_ = port;
 	}
 
-	if (!hostname.empty() && fz::get_address_type(hostname) == fz::address_type::unknown) {
-		auto const utf8 = fz::to_utf8(hostname);
+	if (!hostname_.empty() && fz::get_address_type(hostname_) == fz::address_type::unknown) {
+		auto const utf8 = fz::to_utf8(hostname_);
 		if (!utf8.empty()) {
 			int res = gnutls_server_name_set(m_session, GNUTLS_NAME_DNS, utf8.c_str(), utf8.size());
 			if (res) {
@@ -650,7 +661,7 @@ int CTlsSocketImpl::ContinueHandshake()
 		if (m_shutdown_requested) {
 			int error = Shutdown();
 			if (!error || error != EAGAIN) {
-				tlsSocket_.m_pEvtHandler->send_event<CSocketEvent>(&tlsSocket_, SocketEventType::close, 0);
+				tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::close, 0);
 			}
 		}
 
@@ -680,19 +691,10 @@ int CTlsSocketImpl::Read(void *buffer, unsigned int len, int& error)
 		return -1;
 	}
 
-	if (m_peekDataLen) {
-		auto min = std::min(len, m_peekDataLen);
-		memcpy(buffer, m_peekData, min);
-
-		if (min == m_peekDataLen) {
-			m_peekDataLen = 0;
-			delete [] m_peekData;
-			m_peekData = 0;
-		}
-		else {
-			memmove(m_peekData, m_peekData + min, m_peekDataLen - min);
-			m_peekDataLen -= min;
-		}
+	if (peekBuffer_) {
+		auto min = std::min(static_cast<size_t>(len), peekBuffer_.size());
+		memcpy(buffer, peekBuffer_.get(), min);
+		peekBuffer_.consume(min);
 
 		TriggerEvents();
 
@@ -757,7 +759,7 @@ int CTlsSocketImpl::Write(const void *buffer, unsigned int len, int& error)
 	int res = gnutls_record_send(m_session, buffer, len);
 
 	while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket)
-		res = gnutls_record_send(m_session, 0, 0);
+		res = gnutls_record_send(m_session, nullptr, 0);
 
 	if (res >= 0) {
 		error = 0;
@@ -794,12 +796,12 @@ void CTlsSocketImpl::TriggerEvents()
 		return;
 
 	if (m_canTriggerRead) {
-		tlsSocket_.m_pEvtHandler->send_event<CSocketEvent>(&tlsSocket_, SocketEventType::read, 0);
+		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::read, 0);
 		m_canTriggerRead = false;
 	}
 
 	if (m_canTriggerWrite) {
-		tlsSocket_.m_pEvtHandler->send_event<CSocketEvent>(&tlsSocket_, SocketEventType::write, 0);
+		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::write, 0);
 		m_canTriggerWrite = false;
 	}
 }
@@ -809,7 +811,7 @@ void CTlsSocketImpl::CheckResumeFailedReadWrite()
 	if (m_lastWriteFailed) {
 		int res = GNUTLS_E_AGAIN;
 		while ((res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) && m_canWriteToSocket) {
-			res = gnutls_record_send(m_session, 0, 0);
+			res = gnutls_record_send(m_session, nullptr, 0);
 		}
 
 		if (res == GNUTLS_E_INTERRUPTED || res == GNUTLS_E_AGAIN) {
@@ -826,30 +828,15 @@ void CTlsSocketImpl::CheckResumeFailedReadWrite()
 		m_canTriggerWrite = true;
 	}
 	if (m_lastReadFailed) {
-		assert(!m_peekData);
-
-		m_peekDataLen = 65536;
-		m_peekData = new char[m_peekDataLen];
-
-		int res = DoCallGnutlsRecordRecv(m_peekData, m_peekDataLen);
+		int res = DoCallGnutlsRecordRecv(peekBuffer_.get(65536), 65536);
 		if (res < 0) {
-			m_peekDataLen = 0;
-			delete [] m_peekData;
-			m_peekData = 0;
 			if (res != GNUTLS_E_INTERRUPTED && res != GNUTLS_E_AGAIN) {
 				Failure(res, true);
 			}
 			return;
 		}
 
-		if (!res) {
-			m_peekDataLen = 0;
-			delete [] m_peekData;
-			m_peekData = 0;
-		}
-		else {
-			m_peekDataLen = res;
-		}
+		peekBuffer_.add(static_cast<size_t>(res));
 
 		m_lastReadFailed = false;
 		m_canTriggerRead = true;
@@ -875,28 +862,26 @@ void CTlsSocketImpl::Failure(int code, bool send_close, std::wstring const& func
 	Uninit();
 
 	if (send_close) {
-		tlsSocket_.m_pEvtHandler->send_event<CSocketEvent>(&tlsSocket_, SocketEventType::close, m_socket_error);
+		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::close, m_socket_error);
 	}
 }
 
 int CTlsSocketImpl::Peek(void *buffer, unsigned int len, int& error)
 {
-	if (m_peekData) {
-		auto min = std::min(len, m_peekDataLen);
-		memcpy(buffer, m_peekData, min);
+	if (peekBuffer_) {
+		auto min = std::min(static_cast<size_t>(len), peekBuffer_.size());
+		memcpy(buffer, peekBuffer_.get(), min);
 
 		error = 0;
 		return min;
 	}
 
-	int read = Read(buffer, len, error);
+	int read = Read(peekBuffer_.get(len), len, error);
 	if (read <= 0) {
 		return read;
 	}
-
-	m_peekDataLen = read;
-	m_peekData = new char[m_peekDataLen];
-	memcpy(m_peekData, buffer, m_peekDataLen);
+	peekBuffer_.add(static_cast<size_t>(read));
+	memcpy(buffer, peekBuffer_.get(), static_cast<size_t>(read));
 
 	return read;
 }
@@ -951,7 +936,7 @@ void CTlsSocketImpl::ContinueShutdown()
 	if (!res) {
 		m_tlsState = CTlsSocket::TlsState::closed;
 
-		tlsSocket_.m_pEvtHandler->send_event<CSocketEvent>(&tlsSocket_, SocketEventType::close, 0);
+		tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::close, 0);
 
 		return;
 	}
@@ -976,7 +961,7 @@ void CTlsSocketImpl::TrustCurrentCert(bool trusted)
 		CheckResumeFailedReadWrite();
 
 		if (m_tlsState == CTlsSocket::TlsState::conn) {
-			tlsSocket_.m_pEvtHandler->send_event<CSocketEvent>(&tlsSocket_, SocketEventType::connection, 0);
+			tlsSocket_.m_pEvtHandler->send_event<fz::socket_event>(&tlsSocket_, fz::socket_event_flag::connection, 0);
 		}
 
 		TriggerEvents();
@@ -1041,7 +1026,7 @@ bool CTlsSocketImpl::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& ou
 	std::wstring subject, issuer;
 
 	size = 0;
-	res = gnutls_x509_crt_get_dn(cert, 0, &size);
+	res = gnutls_x509_crt_get_dn(cert, nullptr, &size);
 	if (size) {
 		char* dn = new char[size + 1];
 		dn[size] = 0;
@@ -1065,7 +1050,7 @@ bool CTlsSocketImpl::ExtractCert(gnutls_x509_crt_t const& cert, CCertificate& ou
 	std::vector<std::wstring> alt_subject_names = GetCertSubjectAltNames(cert);
 
 	size = 0;
-	res = gnutls_x509_crt_get_issuer_dn(cert, 0, &size);
+	res = gnutls_x509_crt_get_issuer_dn(cert, nullptr, &size);
 	if (size) {
 		char* dn = new char[++size + 1];
 		dn[size] = 0;
@@ -1131,7 +1116,7 @@ std::vector<std::wstring> CTlsSocketImpl::GetCertSubjectAltNames(gnutls_x509_crt
 	char san[4096];
 	for (unsigned int i = 0; i < 10000; ++i) { // I assume this is a sane limit
 		size_t san_size = sizeof(san) - 1;
-		int const type_or_error = gnutls_x509_crt_get_subject_alt_name(cert, i, san, &san_size, 0);
+		int const type_or_error = gnutls_x509_crt_get_subject_alt_name(cert, i, san, &san_size, nullptr);
 		if (type_or_error == GNUTLS_E_SHORT_MEMORY_BUFFER) {
 			continue;
 		}
@@ -1146,7 +1131,7 @@ std::vector<std::wstring> CTlsSocketImpl::GetCertSubjectAltNames(gnutls_x509_crt
 			}
 		}
 		else if (type_or_error == GNUTLS_SAN_IPADDRESS) {
-			std::wstring ip = fz::to_wstring(CSocket::AddressToString(san, san_size));
+			std::wstring ip = fz::to_wstring(fz::socket::address_to_string(san, san_size));
 			if (!ip.empty()) {
 				ret.emplace_back(std::move(ip));
 			}
@@ -1217,7 +1202,7 @@ int CTlsSocketImpl::GetAlgorithmWarnings()
 
 bool CTlsSocketImpl::GetSortedPeerCertificates(gnutls_x509_crt_t *& certs, unsigned int & certs_size)
 {
-	certs = 0;
+	certs = nullptr;
 	certs_size = 0;
 
 	// First get unsorted list of peer certificates in DER
@@ -1275,6 +1260,49 @@ bool CTlsSocketImpl::GetSortedPeerCertificates(gnutls_x509_crt_t *& certs, unsig
 	return true;
 }
 
+void CTlsSocketImpl::PrintVerificationError(int status)
+{
+	gnutls_datum_t buffer{};
+	gnutls_certificate_verification_status_print(status, GNUTLS_CRT_X509, &buffer, 0);
+	if (buffer.data) {
+		m_pOwner->LogMessage(MessageType::Debug_Warning, L"Gnutls Verification status: %s", buffer.data);
+		gnutls_free(buffer.data);
+	}
+
+	if (status & GNUTLS_CERT_REVOKED) {
+		m_pOwner->LogMessage(MessageType::Error, _("Beware! Certificate has been revoked"));
+
+		// The remaining errors are no longer of interest
+		return;
+	}
+	if (status & GNUTLS_CERT_SIGNATURE_FAILURE) {
+		m_pOwner->LogMessage(MessageType::Error, _("Certificate signature verification failed"));
+		status &= ~GNUTLS_CERT_SIGNATURE_FAILURE;
+	}
+	if (status & GNUTLS_CERT_INSECURE_ALGORITHM) {
+		m_pOwner->LogMessage(MessageType::Error, _("A certificate in the chain was signed using an insecure algorithm"));
+		status &= ~GNUTLS_CERT_INSECURE_ALGORITHM;
+	}
+	if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
+		m_pOwner->LogMessage(MessageType::Error, _("An issuer in the certificate chain is not a certificate authority"));
+		status &= ~GNUTLS_CERT_SIGNER_NOT_CA;
+	}
+	if (status & GNUTLS_CERT_UNEXPECTED_OWNER) {
+		m_pOwner->LogMessage(MessageType::Error, _("The server's hostname does not match the certificate's hostname"));
+		status &= ~GNUTLS_CERT_UNEXPECTED_OWNER;
+	}
+#ifdef GNUTLS_CERT_MISSING_OCSP_STATUS
+	if (status & GNUTLS_CERT_MISSING_OCSP_STATUS) {
+		m_pOwner->LogMessage(MessageType::Error, _("The certificate requires the server to include an OCSP status in its response, but the OCSP status is missing."));
+		status &= ~GNUTLS_CERT_MISSING_OCSP_STATUS;
+	}
+#endif
+	if (status) {
+		m_pOwner->LogMessage(MessageType::Error, _("Received certificate chain could not be verified. Verification status is %d."), status);
+	}
+
+}
+
 int CTlsSocketImpl::VerifyCertificate()
 {
 	if (m_tlsState != CTlsSocket::TlsState::handshake) {
@@ -1333,29 +1361,20 @@ int CTlsSocketImpl::VerifyCertificate()
 	}
 
 	if (status != 0) {
-		if (status & GNUTLS_CERT_REVOKED) {
-			m_pOwner->LogMessage(MessageType::Error, _("Beware! Certificate has been revoked"));
-		}
-		else if (status & GNUTLS_CERT_SIGNATURE_FAILURE) {
-			m_pOwner->LogMessage(MessageType::Error, _("Certificate signature verification failed"));
-		}
-		else if (status & GNUTLS_CERT_INSECURE_ALGORITHM) {
-			m_pOwner->LogMessage(MessageType::Error, _("A certificate in the chain was signed using an insecure algorithm"));
-		}
-		else if (status & GNUTLS_CERT_SIGNER_NOT_CA) {
-			m_pOwner->LogMessage(MessageType::Error, _("An issuer in the certificate chain is not a certificate authority"));
-		}
-		else if (status & GNUTLS_CERT_UNEXPECTED_OWNER) {
-			m_pOwner->LogMessage(MessageType::Error, _("The server's hostname does not match the certificate's hostname"));
-		}
-		else {
-			m_pOwner->LogMessage(MessageType::Error, _("Received certificate chain could not be verified. Verification status is %d."), status);
-		}
+		PrintVerificationError(status);
 
 		Failure(0, true);
 		return FZ_REPLY_ERROR;
 	}
 
+	bool hostnameMismatch = false;
+	if (!hostname_.empty() && fz::get_address_type(hostname_) == fz::address_type::unknown) {
+		if (!gnutls_x509_crt_check_hostname(certs.certs[0], fz::to_utf8(hostname_).c_str())) {
+			hostnameMismatch = true;
+			m_pOwner->LogMessage(MessageType::Debug_Warning, L"Hostname does not match certificate SANs");
+		}
+	}
+
 	datum_holder cert_der{};
 	if (gnutls_x509_crt_export2(certs.certs[0], GNUTLS_X509_FMT_DER, &cert_der) != GNUTLS_E_SUCCESS) {
 		Failure(0, true);
@@ -1373,8 +1392,9 @@ int CTlsSocketImpl::VerifyCertificate()
 
 		TrustCurrentCert(true);
 
-		if (m_tlsState != CTlsSocket::TlsState::conn)
+		if (m_tlsState != CTlsSocket::TlsState::conn) {
 			return FZ_REPLY_ERROR;
+		}
 		return FZ_REPLY_OK;
 	}
 
@@ -1401,14 +1421,15 @@ int CTlsSocketImpl::VerifyCertificate()
 	int const algorithmWarnings = GetAlgorithmWarnings();
 
 	CCertificateNotification *pNotification = new CCertificateNotification(
-		m_pOwner->GetCurrentServer().GetHost(),
-		m_pOwner->GetCurrentServer().GetPort(),
+		fz::to_wstring(hostname_),
+		port_,
 		GetProtocolName(),
 		GetKeyExchange(),
 		GetCipherName(),
 		GetMacName(),
 		algorithmWarnings,
-		std::move(certificates));
+		std::move(certificates),
+		hostnameMismatch);
 
 	// Finally, ask user to verify the certificate chain
 	m_pOwner->SendAsyncRequest(pNotification);
@@ -1493,7 +1514,7 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 	auto list = fz::sprintf("Ciphers for %s:\n", priority);
 
 	gnutls_priority_t pcache;
-	const char *err = 0;
+	const char *err = nullptr;
 	int ret = gnutls_priority_init(&pcache, priority.c_str(), &err);
 	if (ret < 0) {
 		list += fz::sprintf("gnutls_priority_init failed with code %d: %s", ret, err ? err : "Unknown error");
@@ -1512,9 +1533,9 @@ std::string CTlsSocketImpl::ListTlsCiphers(std::string priority)
 
 			gnutls_protocol_t version;
 			unsigned char id[2];
-			const char* name = gnutls_cipher_suite_info(idx, id, NULL, NULL, NULL, &version);
+			const char* name = gnutls_cipher_suite_info(idx, id, nullptr, nullptr, nullptr, &version);
 
-			if (name != 0) {
+			if (name != nullptr) {
 				list += fz::sprintf(
 					"%-50s    0x%02x, 0x%02x    %s\n",
 					name,
@@ -1547,7 +1568,7 @@ int CTlsSocketImpl::DoCallGnutlsRecordRecv(void* data, size_t len)
 
 std::wstring CTlsSocketImpl::GetGnutlsVersion()
 {
-	const char* v = gnutls_check_version(0);
+	const char* v = gnutls_check_version(nullptr);
 	if (!v || !*v) {
 		return L"unknown";
 	}
