@@ -2,6 +2,7 @@
 
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
+#include "view.h"
 #include "LocalListView.h"
 #include "queue.h"
 #include "filezillaapp.h"
@@ -28,7 +29,7 @@
 #include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/recursive_remove.hpp>
 
-class CLocalListViewDropTarget : public CScrollableDropTarget<wxListCtrlEx>
+class CLocalListViewDropTarget final : public CScrollableDropTarget<wxListCtrlEx>
 {
 public:
 	CLocalListViewDropTarget(CLocalListView* pLocalListView)
@@ -114,7 +115,7 @@ public:
 				return wxDragNone;
 			}
 
-			if (!m_pLocalListView->m_state.GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalListView->m_state.GetServer())) {
+			if (!m_pLocalListView->m_state.GetServer() || m_pRemoteDataObject->GetServer().server != m_pLocalListView->m_state.GetServer().server) {
 				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
@@ -132,8 +133,9 @@ public:
 		CScrollableDropTarget<wxListCtrlEx>::OnDrop(x, y);
 		ClearDropHighlight();
 
-		if (m_pLocalListView->m_fileData.empty())
+		if (m_pLocalListView->m_fileData.empty()) {
 			return false;
+		}
 
 		return true;
 	}
@@ -150,33 +152,33 @@ public:
 
 		int flags;
 		int hit = m_pLocalListView->HitTest(point, flags, 0);
-		if (!(flags & wxLIST_HITTEST_ONITEM))
+		if (!(flags & wxLIST_HITTEST_ONITEM)) {
 			hit = -1;
+		}
 
-		if (hit != -1)
-		{
+		if (hit != -1) {
 			const CLocalFileData* const data = m_pLocalListView->GetData(hit);
-			if (!data || !data->dir)
+			if (!data || !data->dir) {
 				hit = -1;
-			else
-			{
+			}
+			else {
 				const CDragDropManager* pDragDropManager = CDragDropManager::Get();
-				if (pDragDropManager && pDragDropManager->pDragSource == m_pLocalListView)
-				{
-					if (m_pLocalListView->GetItemState(hit, wxLIST_STATE_SELECTED))
+				if (pDragDropManager && pDragDropManager->pDragSource == m_pLocalListView) {
+					if (m_pLocalListView->GetItemState(hit, wxLIST_STATE_SELECTED)) {
 						hit = -1;
-					else
+					}
+					else {
 						subDir = data->name;
+					}
 				}
-				else
+				else {
 					subDir = data->name;
+				}
 			}
 		}
-		if (hit != m_pLocalListView->m_dropTarget)
-		{
+		if (hit != m_pLocalListView->m_dropTarget) {
 			ClearDropHighlight();
-			if (hit != -1)
-			{
+			if (hit != -1) {
 				m_pLocalListView->m_dropTarget = hit;
 #ifdef __WXMSW__
 				m_pLocalListView->SetItemState(hit, wxLIST_STATE_DROPHILITED, wxLIST_STATE_DROPHILITED);
@@ -269,9 +271,10 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CLocalListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQueue)
+CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue)
 	: CFileListCtrl<CLocalFileData>(pParent, pQueue),
-	CStateEventHandler(state)
+	CStateEventHandler(state),
+	m_parentView(pParent)
 {
 	wxGetApp().AddStartupProfileRecord("CLocalListView::CLocalListView");
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_DIR);
@@ -279,8 +282,6 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQu
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
 	m_state.RegisterHandler(this, STATECHANGE_SERVER);
 
-	m_dropTarget = -1;
-
 	const unsigned long widths[4] = { 120, 80, 100, 120 };
 
 	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
@@ -291,10 +292,6 @@ CLocalListView::CLocalListView(wxWindow* pParent, CState& state, CQueueView *pQu
 
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
-#ifdef __WXMSW__
-	m_pVolumeEnumeratorThread = 0;
-#endif
-
 	InitHeaderSortImageList();
 
 	InitSort(OPTION_LOCALFILELIST_SORTORDER);
@@ -312,13 +309,13 @@ CLocalListView::~CLocalListView()
 	COptions::Get()->SetOption(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
 
 #ifdef __WXMSW__
-	delete m_pVolumeEnumeratorThread;
+	volumeEnumeratorThread_.reset();
 #endif
 }
 
 bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 {
-	
+
 	CancelLabelEdit();
 
 	wxString focused;
@@ -387,7 +384,7 @@ bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 #ifdef __WXMSW__
 regular_dir:
 #endif
-		CFilterManager filter;
+		CStateFilterManager const& filter = m_state.GetStateFilterManager();
 		fz::local_filesys local_filesys;
 
 		if (!local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false)) {
@@ -406,32 +403,37 @@ regular_dir:
 		bool wasLink;
 		fz::native_string name;
 		while (local_filesys.get_next_file(name, wasLink, data.dir, &data.size, &data.time, &data.attributes)) {
-			if (name.empty()) {
+			data.name = fz::to_wstring(name);
+			if (name.empty() || data.name.empty()) {
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			data.name = fz::to_wstring(name);
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
-				if (data.dir)
+				if (data.dir) {
 					totalDirCount++;
+				}
 				else {
-					if (data.size != -1)
+					if (data.size != -1) {
 						totalSize += data.size;
-					else
+					}
+					else {
 						unknown_sizes++;
+					}
 					totalFileCount++;
 				}
 				m_indexMapping.push_back(num);
 			}
-			else
+			else {
 				hidden++;
+			}
 			num++;
 		}
 
-		if (m_pFilelistStatusBar)
+		if (m_pFilelistStatusBar) {
 			m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
+		}
 	}
 
 	if (m_dropTarget != -1) {
@@ -443,8 +445,9 @@ regular_dir:
 	}
 
 	const int count = m_indexMapping.size();
-	if (oldItemCount != count)
+	if (oldItemCount != count) {
 		SetItemCount(count);
+	}
 
 	SortList(-1, -1, false);
 
@@ -465,21 +468,23 @@ int CLocalListView::OnGetItemImage(long item) const
 {
 	CLocalListView *pThis = const_cast<CLocalListView *>(this);
 	CLocalFileData *data = pThis->GetData(item);
-	if (!data)
+	if (!data) {
 		return -1;
+	}
 	int &icon = data->icon;
 
-	if (icon == -2)
-	{
+	if (icon == -2) {
 		wxString path = _T("");
-		if (data->name != _T(".."))
-		{
+		if (data->name != _T("..")) {
 #ifdef __WXMSW__
-			if (m_dir.GetPath() == _T("\\"))
+			if (m_dir.GetPath() == _T("\\")) {
 				path = data->name + _T("\\");
+			}
 			else
 #endif
+			{
 				path = m_dir.GetPath() + data->name;
+			}
 		}
 
 		icon = pThis->GetIconIndex(data->dir ? iconType::dir : iconType::file, path);
@@ -570,8 +575,8 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -584,7 +589,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 
 	const bool queue_only = action == 1;
 
-	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, *pServer, data->size);
+	m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, path, server, data->size);
 	m_pQueue->QueueFile_Finish(true);
 }
 
@@ -632,7 +637,7 @@ void CLocalListView::DisplayDrives()
 
 		CLocalFileData data;
 		data.name = drive;
-		data.label = fz::sparse_optional<wxString>(data.name);
+		data.label = fz::sparse_optional<std::wstring>(data.name);
 		data.dir = true;
 		data.size = -1;
 		data.icon = GetIconIndex(iconType::dir, _T(""), false);
@@ -642,14 +647,14 @@ void CLocalListView::DisplayDrives()
 		count++;
 	}
 
-	if (m_pFilelistStatusBar)
+	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->SetDirectoryContents(0, drives.size(), 0, false, 0);
+	}
 
-	if (!m_pVolumeEnumeratorThread) {
-		m_pVolumeEnumeratorThread = new CVolumeDescriptionEnumeratorThread(this);
-		if (m_pVolumeEnumeratorThread->Failed()) {
-			delete m_pVolumeEnumeratorThread;
-			m_pVolumeEnumeratorThread = 0;
+	if (!volumeEnumeratorThread_) {
+		volumeEnumeratorThread_ = std::make_unique<CVolumeDescriptionEnumeratorThread>(this);
+		if (volumeEnumeratorThread_->Failed()) {
+			volumeEnumeratorThread_.reset();
 		}
 	}
 }
@@ -666,30 +671,31 @@ void CLocalListView::DisplayShares(wxString computer)
 	DWORD read, total;
 	DWORD resume_handle = 0;
 
-	if (!computer.empty() && computer.Last() == '\\')
+	if (!computer.empty() && computer.Last() == '\\') {
 		computer.RemoveLast();
+	}
 
 	int j = m_fileData.size();
 	int share_count = 0;
 	int res = 0;
-	do
-	{
+	do {
 		const wxWX2WCbuf buf = computer.wc_str(wxConvLocal);
 		res = NetShareEnum((wchar_t*)(const wchar_t*)buf, 1, &si.pShareInfoBlob, MAX_PREFERRED_LENGTH, &read, &total, &resume_handle);
 
-		if (res != ERROR_SUCCESS && res != ERROR_MORE_DATA)
+		if (res != ERROR_SUCCESS && res != ERROR_MORE_DATA) {
 			break;
+		}
 
 		SHARE_INFO_1* p = si.pShareInfo;
-		for (unsigned int i = 0; i < read; i++, p++)
-		{
-			if (p->shi1_type != STYPE_DISKTREE)
+		for (unsigned int i = 0; i < read; ++i, ++p) {
+			if (p->shi1_type != STYPE_DISKTREE) {
 				continue;
+			}
 
 			CLocalFileData data;
 			data.name = p->shi1_netname;
 #ifdef __WXMSW__
-			data.label = fz::sparse_optional<wxString>(data.name);
+			data.label = fz::sparse_optional<std::wstring>(data.name);
 #endif
 			data.dir = true;
 			data.size = -1;
@@ -704,56 +710,68 @@ void CLocalListView::DisplayShares(wxString computer)
 	}
 	while (res == ERROR_MORE_DATA);
 
-	if (m_pFilelistStatusBar)
+	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->SetDirectoryContents(0, share_count, 0, false, 0);
+	}
 }
 
 #endif //__WXMSW__
 
 CLocalFileData* CLocalListView::GetData(unsigned int item)
 {
-	if (!IsItemValid(item))
+	if (!IsItemValid(item)) {
 		return 0;
+	}
 
 	return &m_fileData[m_indexMapping[item]];
 }
 
 bool CLocalListView::IsItemValid(unsigned int item) const
 {
-	if (item >= m_indexMapping.size())
+	if (item >= m_indexMapping.size()) {
 		return false;
+	}
 
 	unsigned int index = m_indexMapping[item];
-	if (index >= m_fileData.size())
+	if (index >= m_fileData.size()) {
 		return false;
+	}
 
 	return true;
 }
 
-CFileListCtrl<CLocalFileData>::CSortComparisonObject CLocalListView::GetSortComparisonObject()
+std::unique_ptr<CFileListCtrlSortBase> CLocalListView::GetSortComparisonObject()
 {
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
 	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
 	if (!m_sortDirection) {
-		if (m_sortColumn == 1)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 2)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 3)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
+		if (m_sortColumn == 1) {
+			return std::make_unique<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 2) {
+			return std::make_unique<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 3) {
+			return std::make_unique<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else {
+			return std::make_unique<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
 	}
 	else {
-		if (m_sortColumn == 1)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 2)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 3)
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
-		else
-			return CFileListCtrl<CLocalFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
+		if (m_sortColumn == 1) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 2) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 3) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else {
+			return std::make_unique<CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+		}
 	}
 }
 
@@ -765,8 +783,9 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	}
 
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_LOCALFILELIST"));
-	if (!pMenu)
+	if (!pMenu) {
 		return;
+	}
 
 	const bool connected = m_state.IsRemoteConnected();
 	if (!connected) {
@@ -787,10 +806,12 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 			pMenu->Enable(XRCID("ID_RENAME"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
 		}
-		if ((data && data->comparison_flags == fill) || (!index && m_hasParent))
+		if ((data && data->comparison_flags == fill) || (!index && m_hasParent)) {
 			fillCount++;
-		if (data && data->dir)
+		}
+		if (data && data->dir) {
 			selectedDir = true;
+		}
 		index = GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
 	if (!count || fillCount == count) {
@@ -807,8 +828,9 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	}
 	else {
 		// Exactly one item selected
-		if (!selectedDir)
+		if (!selectedDir) {
 			pMenu->Delete(XRCID("ID_ENTER"));
+		}
 	}
 	if (selectedDir) {
 		pMenu->Enable(XRCID("ID_EDIT"), false);
@@ -824,8 +846,8 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 
 void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 {
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -845,17 +867,21 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			continue;
-		if (item == -1)
+		}
+		if (item == -1) {
 			break;
+		}
 
 		const CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			break;
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		CServerPath remotePath = m_state.GetRemotePath();
 		if (remotePath.empty()) {
@@ -875,13 +901,14 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 			root.add_dir_to_visit(localPath, remotePath);
 		}
 		else {
-			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, *pServer, data->size);
+			m_pQueue->QueueFile(queue_only, false, data->name, wxEmptyString, m_dir, remotePath, server, data->size);
 			added = true;
 		}
 	}
 
-	if (added)
+	if (added) {
 		m_pQueue->QueueFile_Finish(!queue_only);
+	}
 
 	if (!root.empty()) {
 		recursiveOperation->AddRecursionRoot(std::move(root));
@@ -961,15 +988,18 @@ void CLocalListView::OnMenuDelete(wxCommandEvent&)
 	std::list<fz::native_string> pathsToDelete;
 	long item = -1;
 	while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			continue;
+		}
 
 		CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			continue;
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		pathsToDelete.push_back(fz::to_native(m_dir.GetPath() + data->name));
 	}
@@ -1041,21 +1071,30 @@ void CLocalListView::OnKeyDown(wxKeyEvent& event)
 		wxCommandEvent cmdEvent;
 		OnMenuMkdir(cmdEvent);
 	}
-	else
+	else if (code == 'F' && event.GetModifiers() == wxMOD_CONTROL) {
+		if (m_parentView) {
+			m_parentView->ShowSearchPanel();
+		}
+	}
+	else {
 		event.Skip();
+	}
 }
 
 bool CLocalListView::OnBeginRename(const wxListEvent& event)
 {
-	if (!m_state.GetLocalDir().IsWriteable())
+	if (!m_state.GetLocalDir().IsWriteable()) {
 		return false;
+	}
 
-	if (event.GetIndex() == 0 && m_hasParent)
+	if (event.GetIndex() == 0 && m_hasParent) {
 		return false;
+	}
 
 	const CLocalFileData * const data = GetData(event.GetIndex());
-	if (!data || data->comparison_flags == fill)
+	if (!data || data->comparison_flags == fill) {
 		return false;
+	}
 
 	return true;
 }
@@ -1063,29 +1102,35 @@ bool CLocalListView::OnBeginRename(const wxListEvent& event)
 bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 {
 	const int index = event.GetIndex();
-	if (!index && m_hasParent)
+	if (!index && m_hasParent) {
 		return false;
+	}
 
-	if (event.GetLabel().empty())
+	if (event.GetLabel().empty()) {
 		return false;
+	}
 
-	if (!m_state.GetLocalDir().IsWriteable())
+	if (!m_state.GetLocalDir().IsWriteable()) {
 		return false;
+	}
 
 	CLocalFileData *const data = GetData(event.GetIndex());
-	if (!data || data->comparison_flags == fill)
+	if (!data || data->comparison_flags == fill) {
 		return false;
+	}
 
 	wxString newname = event.GetLabel();
 #ifdef __WXMSW__
 	newname = newname.Left(255);
 #endif
 
-	if (newname == data->name)
+	if (newname == data->name) {
 		return false;
+	}
 
-	if (!RenameFile(this, m_dir.GetPath(), data->name, newname))
+	if (!RenameFile(this, m_dir.GetPath(), data->name, newname)) {
 		return false;
+	}
 
 	data->name = newname;
 #ifdef __WXMSW__
@@ -1098,8 +1143,8 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 
 void CLocalListView::ApplyCurrentFilter()
 {
-	CFilterManager filter;
-
+	CStateFilterManager const& filter = m_state.GetStateFilterManager();
+	
 	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing()) {
 		ExitComparisonMode();
 	}
@@ -1123,24 +1168,29 @@ void CLocalListView::ApplyCurrentFilter()
 	int hidden = 0;
 
 	m_indexMapping.clear();
-	if (m_hasParent)
+	if (m_hasParent) {
 		m_indexMapping.push_back(0);
+	}
 	for (unsigned int i = min; i < m_fileData.size(); ++i) {
 		const CLocalFileData& data = m_fileData[i];
-		if (data.comparison_flags == fill)
+		if (data.comparison_flags == fill) {
 			continue;
+		}
 		if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 			++hidden;
 			continue;
 		}
 
-		if (data.dir)
+		if (data.dir) {
 			++totalDirCount;
+		}
 		else {
-			if (data.size != -1)
+			if (data.size != -1) {
 				totalSize += data.size;
-			else
+			}
+			else {
 				++unknown_sizes;
+			}
 			++totalFileCount;
 		}
 
@@ -1148,8 +1198,9 @@ void CLocalListView::ApplyCurrentFilter()
 	}
 	SetItemCount(m_indexMapping.size());
 
-	if (m_pFilelistStatusBar)
+	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->SetDirectoryContents(totalFileCount, totalDirCount, totalSize, unknown_sizes, hidden);
+	}
 
 	SortList(-1, -1, false);
 
@@ -1160,8 +1211,9 @@ void CLocalListView::ApplyCurrentFilter()
 
 	ReselectItems(selectedNames, focused);
 
-	if (!IsComparing())
+	if (!IsComparing()) {
 		RefreshListOnly();
+	}
 }
 
 std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
@@ -1174,29 +1226,30 @@ std::list<wxString> CLocalListView::RememberSelectedItems(wxString& focused)
 #endif
 	{
 		int item = -1;
-		for (;;)
-		{
+		for (;;) {
 			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-			if (item == -1)
+			if (item < 0 || static_cast<size_t>(item) >= m_indexMapping.size()) {
 				break;
+			}
 			const CLocalFileData &data = m_fileData[m_indexMapping[item]];
-			if (data.comparison_flags != fill)
-			{
-				if (data.dir)
+			if (data.comparison_flags != fill) {
+				if (data.dir) {
 					selectedNames.push_back(_T("d") + data.name);
-				else
+				}
+				else {
 					selectedNames.push_back(_T("-") + data.name);
+				}
 			}
 			SetSelection(item, false);
 		}
 	}
 
 	int item = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
-	if (item != -1)
-	{
+	if (item >= 0 && static_cast<size_t>(item) < m_indexMapping.size()) {
 		const CLocalFileData &data = m_fileData[m_indexMapping[item]];
-		if (data.comparison_flags != fill)
+		if (data.comparison_flags != fill) {
 			focused = data.name;
+		}
 
 		SetItemState(item, 0, wxLIST_STATE_FOCUSED);
 	}
@@ -1209,18 +1262,17 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	// Reselect previous items if neccessary.
 	// Sorting direction did not change. We just have to scan through items once
 
-	if (selectedNames.empty())
-	{
-		if (focused.empty())
+	if (selectedNames.empty()) {
+		if (focused.empty()) {
 			return;
-		for (unsigned int i = 0; i < m_indexMapping.size(); i++)
-		{
+		}
+		for (unsigned int i = 0; i < m_indexMapping.size(); ++i) {
 			const CLocalFileData &data = m_fileData[m_indexMapping[i]];
-			if (data.name == focused)
-			{
+			if (data.name == focused) {
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
-				if (ensureVisible)
+				if (ensureVisible) {
 					EnsureVisible(i);
+				}
 				return;
 			}
 		}
@@ -1230,39 +1282,40 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 	int firstSelected = -1;
 
 	int i = -1;
-	for (std::list<wxString>::const_iterator iter = selectedNames.begin(); iter != selectedNames.end(); ++iter)
-	{
-		while (++i < (int)m_indexMapping.size())
-		{
+	for (auto const& selectedName : selectedNames) {
+		while (++i < (int)m_indexMapping.size()) {
 			const CLocalFileData &data = m_fileData[m_indexMapping[i]];
-			if (data.name == focused)
-			{
+			if (data.name == focused) {
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
-				if (ensureVisible)
+				if (ensureVisible) {
 					EnsureVisible(i);
-				focused = _T("");
+				}
+				focused.clear();
 			}
-			if (data.dir && *iter == (_T("d") + data.name))
-			{
-				if (firstSelected == -1)
+			if (data.dir && selectedName == (_T("d") + data.name)) {
+				if (firstSelected == -1) {
 					firstSelected = i;
-				if (m_pFilelistStatusBar)
+				}
+				if (m_pFilelistStatusBar) {
 					m_pFilelistStatusBar->SelectDirectory();
+				}
 				SetSelection(i, true);
 				break;
 			}
-			else if (*iter == (_T("-") + data.name))
-			{
-				if (firstSelected == -1)
+			else if (selectedName == (_T("-") + data.name)) {
+				if (firstSelected == -1) {
 					firstSelected = i;
-				if (m_pFilelistStatusBar)
+				}
+				if (m_pFilelistStatusBar) {
 					m_pFilelistStatusBar->SelectFile(data.size);
+				}
 				SetSelection(i, true);
 				break;
 			}
 		}
-		if (i == (int)m_indexMapping.size())
+		if (i == (int)m_indexMapping.size()) {
 			break;
+		}
 	}
 	if (!focused.empty()) {
 		if (firstSelected != -1) {
@@ -1276,10 +1329,12 @@ void CLocalListView::ReselectItems(const std::list<wxString>& selectedNames, wxS
 
 void CLocalListView::OnStateChange(t_statechange_notifications notification, const wxString& data, const void*)
 {
-	if (notification == STATECHANGE_LOCAL_DIR)
+	if (notification == STATECHANGE_LOCAL_DIR) {
 		DisplayDir(m_state.GetLocalDir());
-	else if (notification == STATECHANGE_APPLYFILTER)
+	}
+	else if (notification == STATECHANGE_APPLYFILTER) {
 		ApplyCurrentFilter();
+	}
 	else if (notification == STATECHANGE_SERVER) {
 		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
 	}
@@ -1298,11 +1353,13 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	long item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			return;
+		}
 	}
 
 	wxFileDataObject obj;
@@ -1314,15 +1371,18 @@ void CLocalListView::OnBeginDrag(wxListEvent&)
 	item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
 		CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			continue;
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		wxString const name = m_dir.GetPath() + data->name;
 		pDragDropManager->m_localFiles.push_back(name);
@@ -1357,15 +1417,17 @@ void CLocalListView::RefreshFile(const wxString& file)
 
 	bool wasLink;
 	fz::local_filesys::type type = fz::local_filesys::get_file_info(fz::to_native(m_dir.GetPath() + file), wasLink, &data.size, &data.time, &data.attributes);
-	if (type == fz::local_filesys::unknown)
+	if (type == fz::local_filesys::unknown) {
 		return;
+	}
 
 	data.name = file;
 	data.dir = type == fz::local_filesys::dir;
 
-	CFilterManager filter;
-	if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time))
+	CStateFilterManager const& filter = m_state.GetStateFilterManager();
+	if (filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 		return;
+	}
 
 	CancelLabelEdit();
 
@@ -1373,8 +1435,9 @@ void CLocalListView::RefreshFile(const wxString& file)
 	unsigned int i = 0;
 	for (auto iter = m_fileData.begin(); iter != m_fileData.end(); ++iter, ++i) {
 		const CLocalFileData& oldData = *iter;
-		if (oldData.name != file)
+		if (oldData.name != file) {
 			continue;
+		}
 
 		// Update file list status bar
 		if (m_pFilelistStatusBar) {
@@ -1386,31 +1449,41 @@ void CLocalListView::RefreshFile(const wxString& file)
 				int item = -1;
 				for (;;) {
 					item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-					if (item == -1)
+					if (item == -1) {
 						break;
-					if (m_indexMapping[item] != i)
+					}
+					if (m_indexMapping[item] != i) {
 						continue;
+					}
 
-					if (oldData.dir)
+					if (oldData.dir) {
 						m_pFilelistStatusBar->UnselectDirectory();
-					else
+					}
+					else {
 						m_pFilelistStatusBar->UnselectFile(oldData.size);
-					if (data.dir)
+					}
+					if (data.dir) {
 						m_pFilelistStatusBar->SelectDirectory();
-					else
+					}
+					else {
 						m_pFilelistStatusBar->SelectFile(data.size);
+					}
 					break;
 				}
 			}
 
-			if (oldData.dir)
+			if (oldData.dir) {
 				m_pFilelistStatusBar->RemoveDirectory();
-			else
+			}
+			else {
 				m_pFilelistStatusBar->RemoveFile(oldData.size);
-			if (data.dir)
+			}
+			if (data.dir) {
 				m_pFilelistStatusBar->AddDirectory();
-			else
+			}
+			else {
 				m_pFilelistStatusBar->AddFile(data.size);
+			}
 		}
 
 		// Update the data
@@ -1422,17 +1495,20 @@ void CLocalListView::RefreshFile(const wxString& file)
 			RefreshComparison();
 		}
 		else {
-			if (m_sortColumn)
+			if (m_sortColumn) {
 				SortList();
+			}
 			RefreshListOnly(false);
 		}
 		return;
 	}
 
-	if (data.dir)
+	if (data.dir) {
 		m_pFilelistStatusBar->AddDirectory();
-	else
+	}
+	else {
 		m_pFilelistStatusBar->AddFile(data.size);
+	}
 
 	wxString focused;
 	std::list<wxString> selectedNames;
@@ -1450,11 +1526,11 @@ void CLocalListView::RefreshFile(const wxString& file)
 
 	// Find correct position in index mapping
 	std::vector<unsigned int>::iterator start = m_indexMapping.begin();
-	if (m_hasParent)
+	if (m_hasParent) {
 		++start;
-	CFileListCtrl<CLocalFileData>::CSortComparisonObject compare = GetSortComparisonObject();
-	std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), index, compare);
-	compare.Destroy();
+	}
+	std::unique_ptr<CFileListCtrlSortBase> compare = GetSortComparisonObject();
+	std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), index, SortPredicate(compare));
 
 	const int item = insertPos - m_indexMapping.begin();
 	m_indexMapping.insert(insertPos, index);
@@ -1476,8 +1552,9 @@ void CLocalListView::RefreshFile(const wxString& file)
 	}
 	else {
 		RefreshComparison();
-		if (m_pFilelistStatusBar)
+		if (m_pFilelistStatusBar) {
 			m_pFilelistStatusBar->UnselectAll();
+		}
 		ReselectItems(selectedNames, focused);
 	}
 }
@@ -1487,12 +1564,14 @@ wxListItemAttr* CLocalListView::OnGetItemAttr(long item) const
 	CLocalListView *pThis = const_cast<CLocalListView *>(this);
 	const CLocalFileData* const data = pThis->GetData((unsigned int)item);
 
-	if (!data)
+	if (!data) {
 		return 0;
+	}
 
 #ifndef __WXMSW__
-	if (item == m_dropTarget)
+	if (item == m_dropTarget) {
 		return &pThis->m_dropHighlightAttribute;
+	}
 #endif
 
 	switch (data->comparison_flags)
@@ -1517,16 +1596,17 @@ void CLocalListView::StartComparison()
 
 	ComparisonRememberSelections();
 
-	if (m_originalIndexMapping.empty())
+	if (m_originalIndexMapping.empty()) {
 		m_originalIndexMapping.swap(m_indexMapping);
-	else
+	}
+	else {
 		m_indexMapping.clear();
+	}
 
 	m_comparisonIndex = -1;
 
 	const CLocalFileData& last = m_fileData[m_fileData.size() - 1];
-	if (last.comparison_flags != fill)
-	{
+	if (last.comparison_flags != fill) {
 		CLocalFileData data;
 		data.dir = false;
 		data.icon = -1;
@@ -1538,12 +1618,14 @@ void CLocalListView::StartComparison()
 
 bool CLocalListView::get_next_file(wxString& name, bool& dir, int64_t& size, fz::datetime& date)
 {
-	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size())
+	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
+	}
 
 	const unsigned int index = m_originalIndexMapping[m_comparisonIndex];
-	if (index >= m_fileData.size())
+	if (index >= m_fileData.size()) {
 		return false;
+	}
 
 	const CLocalFileData& data = m_fileData[index];
 
@@ -1564,8 +1646,9 @@ void CLocalListView::FinishComparison()
 	RefreshListOnly();
 
 	CComparableListing* pOther = GetOther();
-	if (!pOther)
+	if (!pOther) {
 		return;
+	}
 
 	pOther->ScrollTopItem(GetTopItem());
 }
@@ -1578,8 +1661,9 @@ bool CLocalListView::CanStartComparison()
 wxString CLocalListView::GetItemText(int item, unsigned int column)
 {
 	CLocalFileData *data = GetData(item);
-	if (!data)
+	if (!data) {
 		return wxString();
+	}
 
 	if (!column) {
 #ifdef __WXMSW__
@@ -1589,20 +1673,25 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 #endif
 	}
 	else if (column == 1) {
-		if (data->size < 0)
+		if (data->size < 0) {
 			return wxString();
-		else
+		}
+		else {
 			return CSizeFormat::Format(data->size);
+		}
 	}
 	else if (column == 2) {
-		if (!item && m_hasParent)
+		if (!item && m_hasParent) {
 			return wxString();
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			return wxString();
+		}
 
-		if (data->fileType.empty())
-			data->fileType = GetType(data->name, data->dir, m_dir.GetPath()).ToStdWstring();
+		if (data->fileType.empty()) {
+			data->fileType = GetType(data->name, data->dir, m_dir.GetPath());
+		}
 
 		return data->fileType;
 	}
@@ -1614,7 +1703,7 @@ wxString CLocalListView::GetItemText(int item, unsigned int column)
 
 void CLocalListView::OnMenuEdit(wxCommandEvent&)
 {
-	CServer server;
+	ServerWithCredentials server;
 	CServerPath path;
 
 	if (!m_state.GetServer()) {
@@ -1624,7 +1713,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 		}
 	}
 	else {
-		server = *m_state.GetServer();
+		server = m_state.GetServer();
 
 		path = m_state.GetRemotePath();
 		if (path.empty()) {
@@ -1643,16 +1732,18 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 		}
 
 		const CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			continue;
+		}
 
 		if (data->dir) {
 			wxBell();
 			return;
 		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		selected_item.push_back({m_dir.GetPath() + data->name, data->size});
 	}
@@ -1679,11 +1770,13 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 		}
 
 		const CLocalFileData *data = GetData(item);
-		if (!data)
+		if (!data) {
 			continue;
+		}
 
-		if (data->comparison_flags == fill)
+		if (data->comparison_flags == fill) {
 			continue;
+		}
 
 		selected_item_list.push_back(*data);
 	}
@@ -1704,8 +1797,9 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 		dlg.SetTitle(_("Confirmation needed"));
 		dlg.AddText(_("You have selected more than 10 files or directories to open, do you really want to continue?"));
 
-		if (!dlg.Run())
+		if (!dlg.Run()) {
 			return;
+		}
 	}
 
 	for (auto const& data : selected_item_list) {
@@ -1742,8 +1836,9 @@ void CLocalListView::OnMenuOpen(wxCommandEvent&)
 			continue;
 		}
 
-		if (wxExecute(cmd))
+		if (wxExecute(cmd)) {
 			continue;
+		}
 
 		wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), fn.GetFullPath()), _("Opening failed"), wxICON_EXCLAMATION);
 	}
@@ -1763,16 +1858,15 @@ int64_t CLocalListView::ItemGetSize(int index) const
 
 void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 {
-	if (!m_pVolumeEnumeratorThread) {
+	if (!volumeEnumeratorThread_) {
 		return;
 	}
 
 	std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo> volumeInfo;
-	volumeInfo = m_pVolumeEnumeratorThread->GetVolumes();
+	volumeInfo = volumeEnumeratorThread_->GetVolumes();
 
 	if (event.GetEventType() == fzEVT_VOLUMESENUMERATED) {
-		delete m_pVolumeEnumeratorThread;
-		m_pVolumeEnumeratorThread = 0;
+		volumeEnumeratorThread_.reset();
 	}
 
 	if (m_dir.GetPath() != _T("\\")) {
@@ -1780,12 +1874,12 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 	}
 
 	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter) {
-		wxString drive = iter->volume;
+		std::wstring drive = iter->volume;
 
 		unsigned int item, index;
 		for (item = m_hasParent ? 1 : 0; item < m_indexMapping.size(); ++item) {
 			index = m_indexMapping[item];
-			if (m_fileData[index].name == drive || m_fileData[index].name.substr(0, drive.Len() + 1) == drive + _T(" ")) {
+			if (m_fileData[index].name == drive || m_fileData[index].name.substr(0, drive.size() + 1) == drive + _T(" ")) {
 				break;
 			}
 		}
@@ -1794,7 +1888,7 @@ void CLocalListView::OnVolumesEnumerated(wxCommandEvent& event)
 		}
 
 		if (!iter->volumeName.empty()) {
-			m_fileData[index].label = fz::sparse_optional<wxString>(drive + _T(" (") + iter->volumeName + _T(")"));
+			m_fileData[index].label = fz::sparse_optional<std::wstring>(drive + _T(" (") + iter->volumeName + _T(")"));
 		}
 		if (iter->icon != -1) {
 			m_fileData[index].icon = iter->icon;
