@@ -2,6 +2,7 @@
 
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
+#include "view.h"
 #include "chmoddialog.h"
 #include "commandqueue.h"
 #include "dndobjects.h"
@@ -25,13 +26,14 @@
 #include <algorithm>
 
 #include <libfilezilla/file.hpp>
+#include <libfilezilla/uri.hpp>
 
 #ifdef __WXMSW__
 #include "shellapi.h"
 #include "commctrl.h"
 #endif
 
-class CRemoteListViewDropTarget : public CScrollableDropTarget<wxListCtrlEx>
+class CRemoteListViewDropTarget final : public CScrollableDropTarget<wxListCtrlEx>
 {
 public:
 	CRemoteListViewDropTarget(CRemoteListView* pRemoteListView)
@@ -67,32 +69,36 @@ public:
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
+		{
 			return def;
+		}
 
-		if (!m_pRemoteListView->m_pDirectoryListing)
+		if (!m_pRemoteListView->m_pDirectoryListing) {
 			return wxDragError;
+		}
 
-		if (!GetData())
+		if (!GetData()) {
 			return wxDragError;
+		}
 
 		CDragDropManager* pDragDropManager = CDragDropManager::Get();
-		if (pDragDropManager)
+		if (pDragDropManager) {
 			pDragDropManager->pDropTarget = m_pRemoteListView;
+		}
 
-		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat())
-		{
+		if (m_pDataObject->GetReceivedFormat() == m_pFileDataObject->GetFormat()) {
 			wxString subdir;
 			int flags = 0;
 			int hit = m_pRemoteListView->HitTest(wxPoint(x, y), flags, 0);
-			if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM))
-			{
+			if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM)) {
 				int index = m_pRemoteListView->GetItemIndex(hit);
-				if (index != -1 && m_pRemoteListView->m_fileData[index].comparison_flags != CComparableListing::fill)
-				{
-					if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
+				if (index != -1 && m_pRemoteListView->m_fileData[index].comparison_flags != CComparableListing::fill) {
+					if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount()) {
 						subdir = _T("..");
-					else if ((*m_pRemoteListView->m_pDirectoryListing)[index].is_dir())
+					}
+					else if ((*m_pRemoteListView->m_pDirectoryListing)[index].is_dir()) {
 						subdir = (*m_pRemoteListView->m_pDirectoryListing)[index].name;
+					}
 				}
 			}
 
@@ -102,14 +108,12 @@ public:
 
 		// At this point it's the remote data object.
 
-		if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
-		{
+		if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId()) {
 			wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 			return wxDragNone;
 		}
 
-		if (!m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteListView->m_state.GetServer()))
-		{
+		if (m_pRemoteDataObject->GetServer().server != m_pRemoteListView->m_state.GetServer().server) {
 			wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 			return wxDragNone;
 		}
@@ -120,8 +124,7 @@ public:
 		int hit = m_pRemoteListView->HitTest(wxPoint(x, y), flags, 0);
 		if (hit != -1 && (flags & wxLIST_HITTEST_ONITEM)) {
 			int index = m_pRemoteListView->GetItemIndex(hit);
-			if (index != -1 && m_pRemoteListView->m_fileData[index].comparison_flags != CComparableListing::fill)
-			{
+			if (index != -1 && m_pRemoteListView->m_fileData[index].comparison_flags != CComparableListing::fill) {
 				if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount()) {
 					subdir = _T("..");
 				}
@@ -143,32 +146,28 @@ public:
 		}
 
 		// Make sure target path is valid
-		if (target == m_pRemoteDataObject->GetServerPath())
-		{
+		if (target == m_pRemoteDataObject->GetServerPath()) {
 			wxMessageBoxEx(_("Source and target of the drop operation are identical"));
 			return wxDragNone;
 		}
 
 		const std::list<CRemoteDataObject::t_fileInfo>& files = m_pRemoteDataObject->GetFiles();
-		for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-		{
+		for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 			const CRemoteDataObject::t_fileInfo& info = *iter;
-			if (info.dir)
-			{
+			if (info.dir) {
 				CServerPath dir = m_pRemoteDataObject->GetServerPath();
 				dir.AddSegment(info.name);
-				if (dir == target)
+				if (dir == target) {
 					return wxDragNone;
-				else if (dir.IsParentOf(target, false))
-				{
+				}
+				else if (dir.IsParentOf(target, false)) {
 					wxMessageBoxEx(_("A directory cannot be dragged into one of its subdirectories."));
 					return wxDragNone;
 				}
 			}
 		}
 
-		for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
-		{
+		for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter) {
 			const CRemoteDataObject::t_fileInfo& info = *iter;
 			m_pRemoteListView->m_state.m_pCommandQueue->ProcessCommand(
 				new CRenameCommand(m_pRemoteDataObject->GetServerPath(), info.name, target, info.name)
@@ -176,9 +175,7 @@ public:
 		}
 
 		// Refresh remote listing
-		m_pRemoteListView->m_state.m_pCommandQueue->ProcessCommand(
-			new CListCommand()
-			);
+		m_pRemoteListView->m_state.m_pCommandQueue->ProcessCommand(new CListCommand());
 
 		return wxDragNone;
 	}
@@ -188,8 +185,9 @@ public:
 		CScrollableDropTarget<wxListCtrlEx>::OnDrop(x, y);
 		ClearDropHighlight();
 
-		if (!m_pRemoteListView->m_pDirectoryListing)
+		if (!m_pRemoteListView->m_pDirectoryListing) {
 			return false;
+		}
 
 		return true;
 	}
@@ -204,34 +202,31 @@ public:
 	{
 		int flags;
 		int hit = m_pRemoteListView->HitTest(point, flags, 0);
-		if (!(flags & wxLIST_HITTEST_ONITEM))
+		if (!(flags & wxLIST_HITTEST_ONITEM)) {
 			hit = -1;
+		}
 
-		if (hit != -1)
-		{
+		if (hit != -1) {
 			int index = m_pRemoteListView->GetItemIndex(hit);
-			if (index == -1 || m_pRemoteListView->m_fileData[index].comparison_flags == CComparableListing::fill)
+			if (index == -1 || m_pRemoteListView->m_fileData[index].comparison_flags == CComparableListing::fill) {
 				hit = -1;
-			else if (index != (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
-			{
-				if (!(*m_pRemoteListView->m_pDirectoryListing)[index].is_dir())
+			}
+			else if (index != (int)m_pRemoteListView->m_pDirectoryListing->GetCount()) {
+				if (!(*m_pRemoteListView->m_pDirectoryListing)[index].is_dir()) {
 					hit = -1;
-				else
-				{
+				}
+				else {
 					const CDragDropManager* pDragDropManager = CDragDropManager::Get();
-					if (pDragDropManager && pDragDropManager->pDragSource == m_pRemoteListView)
-					{
+					if (pDragDropManager && pDragDropManager->pDragSource == m_pRemoteListView) {
 						if (m_pRemoteListView->GetItemState(hit, wxLIST_STATE_SELECTED))
 							hit = -1;
 					}
 				}
 			}
 		}
-		if (hit != m_pRemoteListView->m_dropTarget)
-		{
+		if (hit != m_pRemoteListView->m_dropTarget) {
 			ClearDropHighlight();
-			if (hit != -1)
-			{
+			if (hit != -1) {
 				m_pRemoteListView->m_dropTarget = hit;
 #ifdef __WXMSW__
 				m_pRemoteListView->SetItemState(hit, wxLIST_STATE_DROPHILITED, wxLIST_STATE_DROPHILITED);
@@ -260,16 +255,18 @@ public:
 			return wxDragNone;
 		}
 
-		const CServer* const pServer = m_pRemoteListView->m_state.GetServer();
-		wxASSERT(pServer);
+		ServerWithCredentials server = m_pRemoteListView->m_state.GetServer();
+		wxASSERT(server);
 
 		int hit = DoDisplayDropHighlight(wxPoint(x, y));
 		const CDragDropManager* pDragDropManager = CDragDropManager::Get();
 
 		if (hit == -1 && pDragDropManager &&
 			pDragDropManager->remoteParent == m_pRemoteListView->m_pDirectoryListing->path &&
-			*pServer == pDragDropManager->server)
+			server == pDragDropManager->server)
+		{
 			return wxDragNone;
+		}
 
 		return wxDragCopy;
 	}
@@ -294,7 +291,7 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-class CInfoText : public wxWindow
+class CInfoText final : public wxWindow
 {
 public:
 	CInfoText(wxWindow* parent)
@@ -377,9 +374,10 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CRemoteListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CRemoteListView::CRemoteListView(wxWindow* pParent, CState& state, CQueueView* pQueue)
+CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQueue)
 	: CFileListCtrl<CGenericFileData>(pParent, pQueue),
-	CStateEventHandler(state)
+	CStateEventHandler(state),
+	m_parentView(pParent)
 {
 	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
@@ -472,7 +470,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 
 	m_indexMapping[0] = pDirectoryListing->GetCount();
 
-	CFilterManager filter;
+	CFilterManager const& filter = m_state.GetStateFilterManager();
 	const wxString path = m_pDirectoryListing->path.GetPath();
 
 	CGenericFileData last = m_fileData.back();
@@ -485,7 +483,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 		added_indexes.reserve(to_add);
 	}
 
-	CFileListCtrl<CGenericFileData>::CSortComparisonObject compare = GetSortComparisonObject();
+	std::unique_ptr<CFileListCtrlSortBase> compare = GetSortComparisonObject();
 	for (unsigned int i = pDirectoryListing->GetCount() - to_add; i < pDirectoryListing->GetCount(); ++i) {
 		const CDirentry& entry = (*pDirectoryListing)[i];
 		CGenericFileData data;
@@ -498,8 +496,9 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 		}
 		m_fileData.push_back(data);
 
-		if (filter.FilenameFiltered(entry.name, path, entry.is_dir(), entry.size, false, 0, entry.time))
+		if (filter.FilenameFiltered(entry.name, path, entry.is_dir(), entry.size, false, 0, entry.time)) {
 			continue;
+		}
 
 		if (m_pFilelistStatusBar) {
 			if (entry.is_dir())
@@ -512,7 +511,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 		std::vector<unsigned int>::iterator start = m_indexMapping.begin();
 		if (m_hasParent)
 			++start;
-		std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), i, compare);
+		std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), i, SortPredicate(compare));
 
 		const int added_index = insertPos - m_indexMapping.begin();
 		m_indexMapping.insert(insertPos, i);
@@ -526,7 +525,6 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 			added_indexes.insert(added_indexes_insert_pos, added_index);
 		}
 	}
-	compare.Destroy();
 
 	m_fileData.push_back(last);
 
@@ -780,7 +778,8 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 
 		const wxString path = m_pDirectoryListing->path.GetPath();
 
-		CFilterManager filter;
+		CFilterManager const& filter = m_state.GetStateFilterManager();
+		
 		for (unsigned int i = 0; i < m_pDirectoryListing->GetCount(); ++i) {
 			const CDirentry& entry = (*m_pDirectoryListing)[i];
 			CGenericFileData data;
@@ -833,7 +832,9 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 				resetDropTarget = true;
 
 		if (resetDropTarget) {
-			SetItemState(m_dropTarget, 0, wxLIST_STATE_DROPHILITED);
+			if (m_dropTarget < GetItemCount()) {
+				SetItemState(m_dropTarget, 0, wxLIST_STATE_DROPHILITED);
+			}
 			m_dropTarget = -1;
 		}
 	}
@@ -935,16 +936,18 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 
 	if (item) {
 		int index = GetItemIndex(item);
-		if (index == -1)
+		if (index == -1) {
 			return;
-		if (m_fileData[index].comparison_flags == fill)
+		}
+		if (m_fileData[index].comparison_flags == fill) {
 			return;
+		}
 
 		CDirentry const& entry = (*m_pDirectoryListing)[index];
 		std::wstring const& name = entry.name;
 
-		const CServer* pServer = m_state.GetServer();
-		if (!pServer) {
+		ServerWithCredentials const& server = m_state.GetServer();
+		if (!server) {
 			wxBell();
 			return;
 		}
@@ -962,7 +965,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 					m_pLinkResolveState->remote_path = m_pDirectoryListing->path;
 					m_pLinkResolveState->link = name;
 					m_pLinkResolveState->local_path = m_state.GetLocalDir();
-					m_pLinkResolveState->server = *pServer;
+					m_pLinkResolveState->server = server;
 				}
 				m_state.ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 			}
@@ -995,11 +998,12 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 			}
 
 			std::wstring localFile = CQueueView::ReplaceInvalidCharacters(name);
-			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
+			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
 				localFile = StripVMSRevision(localFile);
+			}
 			m_pQueue->QueueFile(queue_only, true, name,
 				(name == localFile) ? std::wstring() : localFile,
-				local_path, m_pDirectoryListing->path, *pServer, entry.size);
+				local_path, m_pDirectoryListing->path, server, entry.size);
 			m_pQueue->QueueFile_Finish(true);
 		}
 	}
@@ -1036,8 +1040,8 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 		CDirentry const& entry = (*m_pDirectoryListing)[index];
 		std::wstring const& name = entry.name;
 
-		const CServer* pServer = m_state.GetServer();
-		if (!pServer) {
+		ServerWithCredentials const& server = m_state.GetServer();
+		if (!server) {
 			wxBell();
 			return;
 		}
@@ -1052,7 +1056,7 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &)
 			m_pLinkResolveState->remote_path = m_pDirectoryListing->path;
 			m_pLinkResolveState->link = name;
 			m_pLinkResolveState->local_path = m_state.GetLocalDir();
-			m_pLinkResolveState->server = *pServer;
+			m_pLinkResolveState->server = server;
 		}
 		m_state.ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 	}
@@ -1069,8 +1073,9 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 	}
 
 	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_REMOTEFILELIST"));
-	if (!pMenu)
+	if (!pMenu) {
 		return;
+	}
 
 	bool const idle = m_state.IsRemoteIdle();
 	bool const userIdle = m_state.IsRemoteIdle(true);
@@ -1102,6 +1107,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
+		pMenu->Enable(XRCID("ID_GETURL_PASSWORD"), false);
 		pMenu->Enable(XRCID("ID_CONTEXT_REFRESH"), false);
 		pMenu->Enable(XRCID("ID_NEW_FILE"), false);
 	}
@@ -1114,6 +1120,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
+		pMenu->Enable(XRCID("ID_GETURL_PASSWORD"), false);
 	}
 	else {
 		if ((GetItemCount() && GetItemState(0, wxLIST_STATE_SELECTED))) {
@@ -1121,6 +1128,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			pMenu->Enable(XRCID("ID_CHMOD"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
 			pMenu->Enable(XRCID("ID_GETURL"), false);
+			pMenu->Enable(XRCID("ID_GETURL_PASSWORD"), false);
 		}
 
 		int count = 0;
@@ -1135,15 +1143,17 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			}
 
 			int index = GetItemIndex(item);
-			if (index == -1)
-				continue;
-			count++;
-			if (m_fileData[index].comparison_flags == fill) {
-				fillCount++;
+			if (index == -1) {
 				continue;
 			}
-			if ((*m_pDirectoryListing)[index].is_dir())
+			++count;
+			if (m_fileData[index].comparison_flags == fill) {
+				++fillCount;
+				continue;
+			}
+			if ((*m_pDirectoryListing)[index].is_dir()) {
 				selectedDir = true;
+			}
 		}
 		if (!count || fillCount == count) {
 			pMenu->Delete(XRCID("ID_ENTER"));
@@ -1154,15 +1164,19 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 			pMenu->Enable(XRCID("ID_CHMOD"), false);
 			pMenu->Enable(XRCID("ID_EDIT"), false);
 			pMenu->Enable(XRCID("ID_GETURL"), false);
+			pMenu->Enable(XRCID("ID_GETURL_PASSWORD"), false);
 		}
 		else {
-			if (selectedDir)
+			if (selectedDir) {
 				pMenu->Enable(XRCID("ID_EDIT"), false);
-			else
+			}
+			else {
 				pMenu->Delete(XRCID("ID_ENTER"));
+			}
 			if (count > 1) {
-				if (selectedDir)
+				if (selectedDir) {
 					pMenu->Delete(XRCID("ID_ENTER"));
+				}
 				pMenu->Enable(XRCID("ID_RENAME"), false);
 			}
 
@@ -1222,8 +1236,8 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 
 	wxASSERT(local_parent.IsWriteable());
 
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
@@ -1234,23 +1248,28 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 	recursion_root root(m_pDirectoryListing->path, false);
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
-		if (!item)
+		}
+		if (!item) {
 			continue;
+		}
 
 		int index = GetItemIndex(item);
-		if (index == -1)
+		if (index == -1) {
 			continue;
-		if (m_fileData[index].comparison_flags == fill)
+		}
+		if (m_fileData[index].comparison_flags == fill) {
 			continue;
+		}
 
 		CDirentry const& entry = (*m_pDirectoryListing)[index];
 		std::wstring const& name = entry.name;
 
 		if (entry.is_dir()) {
-			if (!idle)
+			if (!idle) {
 				continue;
+			}
 			CLocalPath local_path(local_parent);
 			local_path.AddSegment(CQueueView::ReplaceInvalidCharacters(name));
 			CServerPath remotePath = m_pDirectoryListing->path;
@@ -1265,12 +1284,13 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 			}
 			m_pQueue->QueueFile(queue_only, true,
 				name, (name == localFile) ? std::wstring() : localFile,
-				local_parent, m_pDirectoryListing->path, *pServer, entry.size);
+				local_parent, m_pDirectoryListing->path, server, entry.size);
 			added = true;
 		}
 	}
-	if (added)
+	if (added) {
 		m_pQueue->QueueFile_Finish(!queue_only);
+	}
 
 	if (!root.empty()) {
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
@@ -1453,7 +1473,7 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 		if (entry.is_dir() && (follow_symlink || !entry.is_link())) {
 			CServerPath remotePath = m_pDirectoryListing->path;
 			if (remotePath.AddSegment(name)) {
-				root.add_dir_to_visit(m_pDirectoryListing->path, name, CLocalPath(), true);;
+				root.add_dir_to_visit(m_pDirectoryListing->path, name, CLocalPath(), true);
 			}
 		}
 		else {
@@ -1467,7 +1487,7 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 
 	if (!root.empty()) {
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
-		
+
 		CFilterManager filter;
 		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete,
 													 filter.GetActiveFilters(), m_pDirectoryListing->path);
@@ -1523,7 +1543,6 @@ void CRemoteListView::OnKeyDown(wxKeyEvent& event)
 
 		wxCommandEvent tmp;
 		OnMenuDelete(tmp);
-		return;
 	}
 	else if (code == WXK_F2) {
 		wxCommandEvent tmp;
@@ -1541,31 +1560,37 @@ void CRemoteListView::OnKeyDown(wxKeyEvent& event)
 	else if (code == 'N' && event.GetModifiers() == (wxMOD_CONTROL | wxMOD_SHIFT)) {
 		MenuMkdir();
 	}
-	else
+	else if (code == 'F' && event.GetModifiers() == wxMOD_CONTROL) {
+		if (m_parentView) {
+			m_parentView->ShowSearchPanel();
+		}
+	}
+	else {
 		event.Skip();
+	}
 }
 
 bool CRemoteListView::OnBeginRename(const wxListEvent& event)
 {
-	if (!m_state.IsRemoteIdle())
-	{
+	if (!m_state.IsRemoteIdle()) {
 		wxBell();
 		return false;
 	}
 
-	if (!m_pDirectoryListing)
-	{
+	if (!m_pDirectoryListing) {
 		wxBell();
 		return false;
 	}
 
 	int item = event.GetIndex();
-	if (!item)
+	if (!item) {
 		return false;
+	}
 
 	int index = GetItemIndex(item);
-	if (index == -1 || m_fileData[index].comparison_flags == fill)
+	if (index == -1 || m_fileData[index].comparison_flags == fill) {
 		return false;
+	}
 
 	return true;
 }
@@ -1609,13 +1634,11 @@ bool CRemoteListView::OnAcceptRename(const wxListEvent& event)
 		}
 
 		// Check if target file already exists
-		for (unsigned int i = 0; i < m_pDirectoryListing->GetCount(); i++)
-		{
-			if (newFile == (*m_pDirectoryListing)[i].name)
-			{
-				if (wxMessageBoxEx(_("Target filename already exists, really continue?"), _("File exists"), wxICON_QUESTION | wxYES_NO) != wxYES)
+		for (unsigned int i = 0; i < m_pDirectoryListing->GetCount(); ++i) {
+			if (newFile == (*m_pDirectoryListing)[i].name) {
+				if (wxMessageBoxEx(_("Target filename already exists, really continue?"), _("File exists"), wxICON_QUESTION | wxYES_NO) != wxYES) {
 					return false;
-
+				}
 				break;
 			}
 		}
@@ -1764,8 +1787,8 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 
 void CRemoteListView::ApplyCurrentFilter()
 {
-	CFilterManager filter;
-
+	CFilterManager const& filter = m_state.GetStateFilterManager();
+	
 	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing()) {
 		ExitComparisonMode();
 	}
@@ -1796,7 +1819,7 @@ void CRemoteListView::ApplyCurrentFilter()
 			++hidden;
 			continue;
 		}
-
+	
 		if (entry.is_dir())
 			++totalDirCount;
 		else {
@@ -2094,13 +2117,13 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 
 	wxDataObjectComposite object;
 
-	CServer const* pServer = m_state.GetServer();
-	if (!pServer)
+	ServerWithCredentials server = m_state.GetServer(); // Make copy doe to DoDragDrop running the event loop
+	if (!server) {
 		return;
-	CServer const server = *pServer;
+	}
 	CServerPath const path = m_pDirectoryListing->path;
 
-	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(*pServer, m_pDirectoryListing->path);
+	CRemoteDataObject *pRemoteDataObject = new CRemoteDataObject(server, m_pDirectoryListing->path);
 
 	CDragDropManager* pDragDropManager = CDragDropManager::Init();
 	pDragDropManager->pDragSource = this;
@@ -2111,12 +2134,14 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 	item = -1;
 	for (;;) {
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-		if (item == -1)
+		if (item == -1) {
 			break;
+		}
 
 		int index = GetItemIndex(item);
-		if (index == -1 || m_fileData[index].comparison_flags == fill)
+		if (index == -1 || m_fileData[index].comparison_flags == fill) {
 			continue;
+		}
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 
 		pRemoteDataObject->AddFile(entry.name, entry.is_dir(), entry.size, entry.is_link());
@@ -2156,9 +2181,9 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 #if FZ3_USESHELLEXT
 	if (ext) {
 		if (!pRemoteDataObject->DidSendData()) {
-			pServer = m_state.GetServer();
+			server = m_state.GetServer();
 			if (!m_state.IsRemoteIdle() ||
-				!pServer || *pServer != server ||
+				!server || server != pRemoteDataObject->GetServer() ||
 				!m_pDirectoryListing || m_pDirectoryListing->path != path)
 			{
 				// Remote listing has changed since drag started
@@ -2172,8 +2197,9 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 			item = -1;
 			for (;;) {
 				item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
-				if (item == -1)
+				if (item == -1) {
 					break;
+				}
 
 				if (!item) {
 					// Can't drag ".."
@@ -2182,8 +2208,9 @@ void CRemoteListView::OnBeginDrag(wxListEvent&)
 				}
 
 				int index = GetItemIndex(item);
-				if (index == -1 || m_fileData[index].comparison_flags == fill)
+				if (index == -1 || m_fileData[index].comparison_flags == fill) {
 					continue;
+				}
 				if ((*m_pDirectoryListing)[index].is_dir() && !idle) {
 					// Drag could result in recursive operation, don't allow at this point
 					wxBell();
@@ -2221,8 +2248,9 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent&)
 		}
 
 		int index = GetItemIndex(item);
-		if (index == -1 || m_fileData[index].comparison_flags == fill)
+		if (index == -1 || m_fileData[index].comparison_flags == fill) {
 			continue;
+		}
 
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		if (entry.is_dir()) {
@@ -2239,9 +2267,8 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent&)
 		return;
 	}
 
-	const CServerPath path = m_pDirectoryListing->path;
-	const CServer server = *m_state.GetServer();
-	pEditHandler->Edit(CEditHandler::remote, selected_items, path, server, this);
+	CServerPath const path = m_pDirectoryListing->path;
+	pEditHandler->Edit(CEditHandler::remote, selected_items, path, m_state.GetServer(), this);
 }
 
 #ifdef __WXDEBUG__
@@ -2375,91 +2402,110 @@ wxListItemAttr* CRemoteListView::OnGetItemAttr(long item) const
 wxString CRemoteListView::GetItemText(int item, unsigned int column)
 {
 	int index = GetItemIndex(item);
-	if (index == -1)
+	if (index == -1) {
 		return wxString();
+	}
 
-	if (!column)
-	{
-		if ((unsigned int)index == m_pDirectoryListing->GetCount())
+	if (!column) {
+		if ((unsigned int)index == m_pDirectoryListing->GetCount()) {
 			return _T("..");
-		else if ((unsigned int)index < m_pDirectoryListing->GetCount())
+		}
+		else if ((unsigned int)index < m_pDirectoryListing->GetCount()) {
 			return (*m_pDirectoryListing)[index].name;
-		else
+		}
+		else {
 			return wxString();
+		}
 	}
-	if (!item)
+	if (!item) {
 		return wxString(); //.. has no attributes
-
-	if ((unsigned int)index >= m_pDirectoryListing->GetCount())
-		return wxString();
-
-	if (column == 1)
-	{
-		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		if (entry.is_dir() || entry.size < 0)
-			return wxString();
-		else
-			return CSizeFormat::Format(entry.size);
 	}
-	else if (column == 2)
-	{
+
+	if ((unsigned int)index >= m_pDirectoryListing->GetCount()) {
+		return wxString();
+	}
+
+	if (column == 1) {
+		const CDirentry& entry = (*m_pDirectoryListing)[index];
+		if (entry.is_dir() || entry.size < 0) {
+			return wxString();
+		}
+		else {
+			return CSizeFormat::Format(entry.size);
+		}
+	}
+	else if (column == 2) {
 		CGenericFileData& data = m_fileData[index];
-		if (data.fileType.empty())
-		{
+		if (data.fileType.empty()) {
 			const CDirentry& entry = (*m_pDirectoryListing)[index];
-			if (m_pDirectoryListing->path.GetType() == VMS)
-				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir()).ToStdWstring();
-			else
-				data.fileType = GetType(entry.name, entry.is_dir()).ToStdWstring();
+			if (m_pDirectoryListing->path.GetType() == VMS) {
+				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir());
+			}
+			else {
+				data.fileType = GetType(entry.name, entry.is_dir());
+			}
 		}
 
 		return data.fileType;
 	}
-	else if (column == 3)
-	{
+	else if (column == 3) {
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		return CTimeFormat::Format(entry.time);
 	}
-	else if (column == 4)
+	else if (column == 4) {
 		return *(*m_pDirectoryListing)[index].permissions;
-	else if (column == 5)
+	}
+	else if (column == 5) {
 		return *(*m_pDirectoryListing)[index].ownerGroup;
+	}
 	return wxString();
 }
 
-CFileListCtrl<CGenericFileData>::CSortComparisonObject CRemoteListView::GetSortComparisonObject()
+std::unique_ptr<CFileListCtrlSortBase> CRemoteListView::GetSortComparisonObject()
 {
 	CFileListCtrlSort<CDirectoryListing>::DirSortMode dirSortMode = GetDirSortMode();
 	CFileListCtrlSort<CDirectoryListing>::NameSortMode nameSortMode = GetNameSortMode();
 
 	CDirectoryListing const& directoryListing = *m_pDirectoryListing;
 	if (!m_sortDirection) {
-		if (m_sortColumn == 1)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 2)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortType<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 3)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortTime<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 4)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortPermissions<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 5)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortOwnerGroup<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CFileListCtrlSortName<CDirectoryListing, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
+		if (m_sortColumn == 1) {
+			return std::make_unique<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 2) {
+			return std::make_unique<CFileListCtrlSortType<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 3) {
+			return std::make_unique<CFileListCtrlSortTime<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 4) {
+			return std::make_unique<CFileListCtrlSortPermissions<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 5) {
+			return std::make_unique<CFileListCtrlSortOwnerGroup<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else {
+			return std::make_unique<CFileListCtrlSortName<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
 	}
 	else {
-		if (m_sortColumn == 1)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 2)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<CDirectoryListing, CGenericFileData>, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 3)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<CDirectoryListing, CGenericFileData>, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 4)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPermissions<CDirectoryListing, CGenericFileData>, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else if (m_sortColumn == 5)
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortOwnerGroup<CDirectoryListing, CGenericFileData>, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
-		else
-			return CFileListCtrl<CGenericFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortName<CDirectoryListing, CGenericFileData>, CGenericFileData>(directoryListing, m_fileData, dirSortMode, nameSortMode, this));
+		if (m_sortColumn == 1) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 2) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortType<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 3) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortTime<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 4) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortPermissions<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else if (m_sortColumn == 5) {
+			return std::make_unique<CReverseSort<CFileListCtrlSortOwnerGroup<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
+		else {
+			return std::make_unique<CReverseSort<CFileListCtrlSortName<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+		}
 	}
 }
 
@@ -2483,8 +2529,9 @@ void CRemoteListView::LinkIsNotDir(CServerPath const& path, std::wstring const&
 {
 	if (m_pLinkResolveState && m_pLinkResolveState->remote_path == path && m_pLinkResolveState->link == link) {
 		std::wstring localFile = CQueueView::ReplaceInvalidCharacters(link);
-		if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
+		if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION)) {
 			localFile = StripVMSRevision(localFile);
+		}
 		m_pQueue->QueueFile(false, true,
 			link, (link != localFile) ? localFile : std::wstring(),
 			m_pLinkResolveState->local_path, m_pLinkResolveState->remote_path, m_pLinkResolveState->server, -1);
@@ -2500,8 +2547,8 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 		return;
 	}
 
-	const CServer* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		return;
 	}
 
@@ -2515,10 +2562,11 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 		}
 
 		int index = GetItemIndex(item);
-		if (index == -1 || m_fileData[index].comparison_flags == fill)
+		if (index == -1 || m_fileData[index].comparison_flags == fill) {
 			continue;
+		}
 
-		const CDirentry& entry = (*m_pDirectoryListing)[index];
+		CDirentry const& entry = (*m_pDirectoryListing)[index];
 
 		selected_item_list.push_back(entry);
 	}
@@ -2534,12 +2582,12 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 
 	const CServerPath& path = m_pDirectoryListing->path;
 
-	wxString const server = pServer->Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
+	wxString const url = server.Format((event.GetId() == XRCID("ID_GETURL_PASSWORD")) ? ServerFormat::url_with_password : ServerFormat::url);
 
 	auto getUrl = [](wxString const& serverPart, CServerPath const& path, std::wstring const& name) {
 		wxString url = serverPart;
 
-		auto const pathPart = url_encode(path.FormatFilename(name, false), true);
+		auto const pathPart = fz::percent_encode_w(path.FormatFilename(name, false), true);
 		if (!pathPart.empty() && pathPart[0] != '/') {
 			url += '/';
 		}
@@ -2550,11 +2598,11 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 
 	wxString urls;
 	if (selected_item_list.size() == 1) {
-		urls = getUrl(server, path, selected_item_list.front().name);
+		urls = getUrl(url, path, selected_item_list.front().name);
 	}
 	else {
 		for (auto const& entry : selected_item_list) {
-			urls += getUrl(server, path, entry.name);
+			urls += getUrl(url, path, entry.name);
 #ifdef __WXMSW__
 			urls += _T("\r\n");
 #else
@@ -2572,13 +2620,16 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 int CRemoteListView::GetOverlayIndex(int item)
 {
 	int index = GetItemIndex(item);
-	if (index == -1)
+	if (index == -1) {
 		return 0;
-	if ((unsigned int)index >= m_pDirectoryListing->GetCount())
+	}
+	if ((unsigned int)index >= m_pDirectoryListing->GetCount()) {
 		return 0;
+	}
 
-	if ((*m_pDirectoryListing)[index].is_link())
+	if ((*m_pDirectoryListing)[index].is_link()) {
 		return GetLinkOverlayIndex();
+	}
 
 	return 0;
 }
@@ -2614,11 +2665,13 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 	}
 
 	CInputDialog dlg;
-	if (!dlg.Create(this, _("Create empty file"), _("Please enter the name of the file which should be created:")))
+	if (!dlg.Create(this, _("Create empty file"), _("Please enter the name of the file which should be created:"))) {
 		return;
+	}
 
-	if (dlg.ShowModal() != wxID_OK)
+	if (dlg.ShowModal() != wxID_OK) {
 		return;
+	}
 
 	if (dlg.GetValue().empty()) {
 		wxBell();
@@ -2646,8 +2699,8 @@ void CRemoteListView::OnMenuNewfile(wxCommandEvent&)
 		(void)f;
 	}
 
-	CServer const* pServer = m_state.GetServer();
-	if (!pServer) {
+	ServerWithCredentials const& server = m_state.GetServer();
+	if (!server) {
 		wxBell();
 		return;
 	}
