@@ -1,5 +1,5 @@
-#ifndef __REMOTELISTVIEW_H__
-#define __REMOTELISTVIEW_H__
+#ifndef FILEZILLA_INTERFACE_REMOTELISTVIEW_HEADER
+#define FILEZILLA_INTERFACE_REMOTELISTVIEW_HEADER
 
 #include "state.h"
 #include "listctrlex.h"
@@ -9,13 +9,14 @@ class CChmodDialog;
 class CInfoText;
 class CQueueView;
 class CRemoteListViewDropTarget;
+class CView;
 class CWindowTinter;
 
-class CRemoteListView : public CFileListCtrl<CGenericFileData>, CStateEventHandler
+class CRemoteListView final : public CFileListCtrl<CGenericFileData>, CStateEventHandler
 {
 	friend class CRemoteListViewDropTarget;
 public:
-	CRemoteListView(wxWindow* pParent, CState& state, CQueueView* pQueue);
+	CRemoteListView(CView* pParent, CState& state, CQueueView* pQueue);
 	virtual ~CRemoteListView();
 
 	virtual bool CanStartComparison();
@@ -50,7 +51,7 @@ protected:
 	bool IsItemValid(unsigned int item) const;
 	int GetItemIndex(unsigned int item) const;
 
-	virtual CSortComparisonObject GetSortComparisonObject();
+	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() override;
 
 	virtual void OnStateChange(t_statechange_notifications notification, const wxString& data, const void* data2);
 	void ApplyCurrentFilter();
@@ -91,7 +92,7 @@ protected:
 	// already.
 	struct t_linkResolveState
 	{
-		CServer server;
+		ServerWithCredentials server;
 		CServerPath remote_path;
 		wxString link;
 		CLocalPath local_path;
@@ -102,6 +103,8 @@ protected:
 
 	std::unique_ptr<CWindowTinter> m_windowTinter;
 
+	CView *m_parentView{};
+	
 	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent &event);
 	void OnContextMenu(wxContextMenuEvent& event);
