@@ -123,8 +123,9 @@ bool CAsyncRequestQueue::AddRequest(CFileZillaEngine *pEngine, std::unique_ptr<C
 
 bool CAsyncRequestQueue::ProcessNextRequest()
 {
-	if (m_requestList.empty())
+	if (m_requestList.empty()) {
 		return true;
+	}
 
 	t_queueEntry &entry = m_requestList.front();
 
@@ -146,23 +147,30 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 		}
 		bool canRemember = notification.GetType() == CInteractiveLoginNotification::keyfile;
 
-		if (CLoginManager::Get().GetPassword(notification.server, true, wxString(), notification.GetChallenge(), canRemember))
+		ServerWithCredentials server(notification.server, notification.credentials);
+		if (CLoginManager::Get().GetPassword(server, true, std::wstring(), notification.GetChallenge(), canRemember)) {
+			notification.credentials = server.credentials;
 			notification.passwordSet = true;
+		}
 		else {
 			// Retry with prompt
 
-			if (!CheckWindowState())
+			if (!CheckWindowState()) {
 				return false;
+			}
 
-			if (CLoginManager::Get().GetPassword(notification.server, false, wxString(), notification.GetChallenge(), canRemember))
+			if (CLoginManager::Get().GetPassword(server, false, std::wstring(), notification.GetChallenge(), canRemember)) {
+				notification.credentials = server.credentials;
 				notification.passwordSet = true;
+			}
 		}
 
 		entry.pEngine->SetAsyncRequestReply(std::move(entry.pNotification));
 	}
 	else if (entry.pNotification->GetRequestID() == reqId_hostkey || entry.pNotification->GetRequestID() == reqId_hostkeyChanged) {
-		if (!CheckWindowState())
+		if (!CheckWindowState()) {
 			return false;
+		}
 
 		auto & notification = static_cast<CHostKeyNotification&>(*entry.pNotification.get());
 
@@ -170,14 +178,16 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 			notification.m_trust = true;
 			notification.m_alwaysTrust = false;
 		}
-		else
+		else {
 			CVerifyHostkeyDialog::ShowVerificationDialog(m_pMainFrame, notification);
+		}
 
 		entry.pEngine->SetAsyncRequestReply(std::move(entry.pNotification));
 	}
 	else if (entry.pNotification->GetRequestID() == reqId_certificate) {
-		if (!CheckWindowState())
+		if (!CheckWindowState()) {
 			return false;
+		}
 
 		auto & notification = static_cast<CCertificateNotification&>(*entry.pNotification.get());
 		m_pVerifyCertDlg->ShowVerificationDialog(notification);
@@ -278,14 +288,16 @@ bool CAsyncRequestQueue::ProcessFileExistsNotification(t_queueEntry &entry)
 				return false;
 
 			wxString msg;
-			wxString defaultName;
+			std::wstring defaultName;
 			if (notification.download) {
 				msg.Printf(_("The file %s already exists.\nPlease enter a new name:"), notification.localFile);
-				wxFileName fn = notification.localFile;
-				defaultName = fn.GetFullName();
+				CLocalPath fn(notification.localFile, &defaultName);
+				if (fn.empty() || defaultName.empty()) {
+					defaultName = _("new name");
+				}
 			}
 			else {
-				wxString fullName = notification.remotePath.GetPath() + notification.remoteFile;
+				wxString fullName = notification.remotePath.FormatFilename(notification.remoteFile);
 				msg.Printf(_("The file %s already exists.\nPlease enter a new name:"), fullName);
 				defaultName = notification.remoteFile;
 			}
