@@ -13,6 +13,24 @@
 
 #include <libfilezilla/local_filesys.hpp>
 
+#include <algorithm>
+
+bool CStateFilterManager::FilenameFiltered(std::wstring const& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const
+{
+	if (local) {
+		if (m_localFilter && FilenameFilteredByFilter(m_localFilter, name, path, dir, size, attributes, date)) {
+			return true;
+		}
+	}
+	else {
+		if (m_remoteFilter && FilenameFilteredByFilter(m_remoteFilter, name, path, dir, size, attributes, date)) {
+			return true;
+		}
+	}
+
+	return CFilterManager::FilenameFiltered(name, path, dir, size, local, attributes, date);
+}
+
 CContextManager CContextManager::m_the_context_manager;
 
 CContextManager::CContextManager()
@@ -181,7 +199,7 @@ void CContextManager::ProcessDirectoryListing(CServer const& server, std::shared
 		if (state == exempt) {
 			continue;
 		}
-		if (state->GetServer() && *state->GetServer() == server) {
+		if (state->GetServer() && state->GetServer().server == server) {
 			state->SetRemoteDir(listing, true);
 		}
 	}
@@ -210,7 +228,7 @@ CState::~CState()
 
 	// Unregister all handlers
 	for (int i = 0; i < STATECHANGE_MAX; ++i) {
-		wxASSERT(m_handlers[i].empty());
+		wxASSERT(m_handlers[i].handlers.empty());
 	}
 }
 
@@ -242,7 +260,7 @@ bool CState::SetLocalDir(std::wstring const& dir, std::wstring *error, bool reme
 bool CState::SetLocalDir(CLocalPath const& dir, std::wstring *error, bool rememberPreviousSubdir)
 {
 	if (m_changeDirFlags.syncbrowse) {
-		wxMessageBoxEx(_T("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
+		wxMessageBoxEx(_("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
 		return false;
 	}
 
@@ -251,7 +269,7 @@ bool CState::SetLocalDir(CLocalPath const& dir, std::wstring *error, bool rememb
 	}
 
 	if (!m_sync_browse.local_root.empty()) {
-		wxASSERT(m_site.m_server);
+		wxASSERT(m_site.server_);
 
 		if (dir != m_sync_browse.local_root && !dir.IsSubdirOf(m_sync_browse.local_root)) {
 			wxString msg = wxString::Format(_("The local directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the local directory?"),
@@ -474,10 +492,10 @@ void CState::LocalDirCreated(const CLocalPath& path)
 
 void CState::SetSite(Site const& site, CServerPath const& path)
 {
-	if (m_site.m_server) {
-		if (site.m_server && site.m_server == m_site.m_server &&
-			site.m_server.GetName() == m_site.m_server.GetName() &&
-			site.m_server.MaximumMultipleConnections() == m_site.m_server.MaximumMultipleConnections())
+	if (m_site.server_) {
+		if (site.server_ && site.server_ == m_site.server_ &&
+			site.server_.server.GetName() == m_site.server_.server.GetName() &&
+			site.server_.server.MaximumMultipleConnections() == m_site.server_.server.MaximumMultipleConnections())
 		{
 			// Nothing changes
 			return;
@@ -487,11 +505,11 @@ void CState::SetSite(Site const& site, CServerPath const& path)
 		m_pCertificate.reset();
 		m_pSftpEncryptionInfo.reset();
 	}
-	if (site.m_server) {
+	if (site.server_) {
 		if (!path.empty()) {
 			m_last_path = path;
 		}
-		else if (m_last_site.m_server != site.m_server) {
+		else if (m_last_site.server_ != site.server_) {
 			m_last_path.clear();
 		}
 		m_last_site = site;
@@ -511,9 +529,9 @@ Site const& CState::GetSite() const
 	return m_site;
 }
 
-CServer const* CState::GetServer() const
+ServerWithCredentials const& CState::GetServer() const
 {
-	return m_site.m_server ? &m_site.m_server : 0;
+	return m_site.server_;
 }
 
 wxString CState::GetTitle() const
@@ -523,7 +541,7 @@ wxString CState::GetTitle() const
 
 bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 {
-	if (!site.m_server) {
+	if (!site.server_) {
 		return false;
 	}
 	if (!m_pEngine) {
@@ -535,7 +553,7 @@ bool CState::Connect(Site const& site, CServerPath const& path, bool compare)
 	m_pRemoteRecursiveOperation->StopRecursiveOperation();
 	SetSyncBrowse(false);
 
-	m_pCommandQueue->ProcessCommand(new CConnectCommand(site.m_server));
+	m_pCommandQueue->ProcessCommand(new CConnectCommand(site.server_.server, site.server_.credentials));
 	m_pCommandQueue->ProcessCommand(new CListCommand(path, _T(""), LIST_FLAG_FALLBACK_CURRENT));
 
 	SetSite(site, path);
@@ -568,8 +586,9 @@ bool CState::Disconnect()
 bool CState::CreateEngine()
 {
 	wxASSERT(!m_pEngine);
-	if (m_pEngine)
+	if (m_pEngine) {
 		return true;
+	}
 
 	m_pEngine = new CFileZillaEngine(m_mainFrame.GetEngineContext(), m_mainFrame);
 
@@ -590,28 +609,30 @@ void CState::RegisterHandler(CStateEventHandler* pHandler, t_statechange_notific
 {
 	wxASSERT(pHandler);
 	wxASSERT(&pHandler->m_state == this);
-	if (!pHandler || &pHandler->m_state != this)
+	if (!pHandler || &pHandler->m_state != this) {
 		return;
+	}
 	wxASSERT(notification != STATECHANGE_MAX && notification != STATECHANGE_NONE);
 	wxASSERT(pHandler != insertBefore);
 
 
 	auto &handlers = m_handlers[notification];
-	auto insertionPoint = handlers.end();
 
-	for (auto it = handlers.begin(); it != handlers.end(); ++it) {
-		if (it->pHandler == insertBefore) {
+	wxASSERT(!insertBefore || !handlers.inNotify_);
+
+	auto insertionPoint = handlers.handlers.end();
+
+	for (auto it = handlers.handlers.begin(); it != handlers.handlers.end(); ++it) {
+		if (*it == insertBefore) {
 			insertionPoint = it;
 		}
-		if (it->pHandler == pHandler) {
-			wxASSERT(insertionPoint == handlers.end());
+		if (*it == pHandler) {
+			wxASSERT(insertionPoint == handlers.handlers.end());
 			return;
 		}
 	}
 
-	t_handler handler;
-	handler.pHandler = pHandler;
-	handlers.insert(insertionPoint, handler);
+	handlers.handlers.insert(insertionPoint, pHandler);
 }
 
 void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notifications notification)
@@ -622,9 +643,15 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notif
 	if (notification == STATECHANGE_NONE) {
 		for (int i = 0; i < STATECHANGE_MAX; ++i) {
 			auto &handlers = m_handlers[i];
-			for (auto iter = handlers.begin(); iter != handlers.end(); ++iter) {
-				if (iter->pHandler == pHandler) {
-					handlers.erase(iter);
+			for (auto iter = handlers.handlers.begin(); iter != handlers.handlers.end(); ++iter) {
+				if (*iter == pHandler) {
+					if (handlers.inNotify_) {
+						handlers.compact_ = true;
+						*iter = 0;
+					}
+					else {
+						handlers.handlers.erase(iter);
+					}
 					break;
 				}
 			}
@@ -632,9 +659,15 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, t_statechange_notif
 	}
 	else {
 		auto &handlers = m_handlers[notification];
-		for (auto iter = handlers.begin(); iter != handlers.end(); ++iter) {
-			if (iter->pHandler == pHandler) {
-				handlers.erase(iter);
+		for (auto iter = handlers.handlers.begin(); iter != handlers.handlers.end(); ++iter) {
+			if (*iter == pHandler) {
+				if (handlers.inNotify_) {
+					handlers.compact_ = true;
+					*iter = 0;
+				}
+				else {
+					handlers.handlers.erase(iter);
+				}
 				return;
 			}
 		}
@@ -645,11 +678,25 @@ void CState::NotifyHandlers(t_statechange_notifications notification, const wxSt
 {
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
-	auto const& handlers = m_handlers[notification];
-	for (auto const& handler : handlers) {
-		handler.pHandler->OnStateChange(notification, data, data2);
+	auto & handlers = m_handlers[notification];
+
+	wxASSERT(!handlers.inNotify_);
+
+	handlers.inNotify_ = true;
+	// Can't use iterators as inserting a handler might invalidate them
+	for (size_t i = 0; i < handlers.handlers.size(); ++i) {
+		if (handlers.handlers[i]) {
+			handlers.handlers[i]->OnStateChange(notification, data, data2);
+		}
 	}
 
+	if (handlers.compact_) {
+		handlers.handlers.erase(std::remove(handlers.handlers.begin(), handlers.handlers.end(), nullptr), handlers.handlers.end());
+		handlers.compact_ = false;
+	}
+
+	handlers.inNotify_ = false;
+
 	CContextManager::Get()->NotifyHandlers(this, notification, data, data2);
 }
 
@@ -670,7 +717,7 @@ CGlobalStateEventHandler::~CGlobalStateEventHandler()
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const wxString& subdir, bool queueOnly)
 {
-	if (!m_site.m_server || !m_pDirectoryListing) {
+	if (!m_site.server_ || !m_pDirectoryListing) {
 		return;
 	}
 
@@ -687,7 +734,7 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const w
 
 void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const CServerPath& path, bool queueOnly)
 {
-	if (!m_site.m_server) {
+	if (!m_site.server_) {
 		return;
 	}
 
@@ -710,7 +757,7 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 		if (type == fz::local_filesys::file) {
 			std::wstring localFile;
 			const CLocalPath localPath(files[i].ToStdWstring(), &localFile);
-			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, m_site.m_server, size);
+			m_mainFrame.GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, m_site.server_, size);
 			m_mainFrame.GetQueue()->QueueFile_Finish(!queueOnly);
 		}
 		else if (type == fz::local_filesys::dir) {
@@ -736,8 +783,9 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const CLocalPath& path, bool copy)
 {
 	const wxArrayString &files = pFileDataObject->GetFilenames();
-	if (!files.Count())
+	if (!files.Count()) {
 		return;
+	}
 
 #ifdef __WXMSW__
 	int len = 1;
@@ -784,34 +832,41 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 				continue;
 			}
 			wxString target = path.GetPath() + name;
-			if (file == target)
+			if (file == target) {
 				continue;
+			}
 
-			if (copy)
+			if (copy) {
 				wxCopyFile(file, target);
-			else
+			}
+			else {
 				wxRenameFile(file, target);
+			}
 		}
 		else if (type == fz::local_filesys::dir) {
 			CLocalPath sourcePath(file);
-			if (sourcePath == path || sourcePath.GetParent() == path)
+			if (sourcePath == path || sourcePath.GetParent() == path) {
 				continue;
+			}
 			if (sourcePath.IsParentOf(path)) {
 				error = _("A directory cannot be dragged into one of its subdirectories.");
 				continue;
 			}
 
-			if (copy)
+			if (copy) {
 				RecursiveCopy(sourcePath, path);
+			}
 			else {
-				if (!sourcePath.HasParent())
+				if (!sourcePath.HasParent()) {
 					continue;
+				}
 				wxRenameFile(file, path.GetPath() + sourcePath.GetLastSegment());
 			}
 		}
 	}
-	if (!error.empty())
+	if (!error.empty()) {
 		wxMessageBoxEx(error, _("Could not complete operation"));
+	}
 #endif
 
 	RefreshLocal();
@@ -936,7 +991,7 @@ bool CState::IsRemoteConnected() const
 		return false;
 	}
 
-	return static_cast<bool>(m_site.m_server);
+	return static_cast<bool>(m_site.server_);
 }
 
 bool CState::IsRemoteIdle(bool ignore_recursive) const
@@ -1004,7 +1059,7 @@ void CState::LinkIsNotDir(const CServerPath& path, const wxString& subdir)
 
 bool CState::ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir, int flags, bool ignore_busy, bool compare)
 {
-	if (!m_site.m_server || !m_pCommandQueue) {
+	if (!m_site.server_ || !m_pCommandQueue) {
 		return false;
 	}
 
@@ -1216,21 +1271,21 @@ void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
 
 void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
 {
-	if (newSite.m_path.empty() || !newSite.m_server) {
+	if (newSite.m_path.empty() || !newSite.server_) {
 		return;
 	}
 
 	bool changed = false;
-	if (m_site.m_server && m_site != newSite) {
-		if (m_site.m_path == oldPath && m_site.m_server.EqualsNoPass(newSite.m_server)) {
+	if (m_site.server_ && m_site != newSite) {
+		if (m_site.m_path == oldPath && m_site.server_ == newSite.server_) {
 			m_site = newSite;
 			changed = true;
 		}
 	}
-	if (m_last_site.m_server && m_last_site != newSite) {
-		if (m_last_site.m_path == oldPath && m_last_site.m_server.EqualsNoPass(newSite.m_server)) {
+	if (m_last_site.server_ && m_last_site != newSite) {
+		if (m_last_site.m_path == oldPath && m_last_site.server_ == newSite.server_) {
 			m_last_site = newSite;
-			if (!m_site.m_server) {
+			if (!m_site.server_) {
 				// Active site has precedence over historic data
 				changed = true;
 			}
@@ -1244,13 +1299,13 @@ void CState::UpdateSite(wxString const& oldPath, Site const& newSite)
 
 void CState::UpdateTitle()
 {
-	if (m_site.m_server) {
-		wxString const& name = m_site.m_server.GetName();
+	if (m_site.server_) {
+		wxString const& name = m_site.server_.server.GetName();
 		m_title.clear();
 		if (!name.empty()) {
 			m_title = name + _T(" - ");
 		}
-		m_title += m_site.m_server.Format(ServerFormat::with_user_and_optional_port);
+		m_title += m_site.server_.Format(ServerFormat::with_user_and_optional_port);
 	}
 	else {
 		m_title = _("Not connected");
