@@ -1,8 +1,9 @@
-#ifndef __STATE_H__
-#define __STATE_H__
+#ifndef FILEZILLA_INTERFACE_STATE_HEADER
+#define FILEZILLA_INTERFACE_STATE_HEADER
 
 #include "local_path.h"
 #include "sitemanager.h"
+#include "filter.h"
 
 #include <memory>
 
@@ -40,6 +41,9 @@ enum t_statechange_notifications
 	STATECHANGE_CHANGEDCONTEXT, /* Currently active context changed */
 	STATECHANGE_REMOVECONTEXT, /* Right before deleting a context */
 	STATECHANGE_GLOBALBOOKMARKS,
+	STATECHANGE_REWRITE_CREDENTIALS,
+
+	STATECHANGE_QUITNOW,
 
 	STATECHANGE_MAX
 };
@@ -55,6 +59,22 @@ class CRemoteDataObject;
 class CRemoteRecursiveOperation;
 class CComparisonManager;
 
+class CStateFilterManager final : public CFilterManager
+{
+public:
+	virtual bool FilenameFiltered(std::wstring const& name, const wxString& path, bool dir, int64_t size, bool local, int attributes, fz::datetime const& date) const override;
+
+	CFilter const& GetLocalFilter() const { return m_localFilter; }
+	void SetLocalFilter(CFilter const& filter) { m_localFilter = filter; }
+
+	CFilter const& GetRemoteFilter() const { return m_remoteFilter; }
+	void SetRemoteFilter(CFilter const& filter) { m_remoteFilter = filter; }
+
+private:
+	CFilter m_localFilter;
+	CFilter m_remoteFilter;
+};
+
 class CState;
 class CContextManager final
 {
@@ -81,6 +101,8 @@ public:
 
 	void SetCurrentContext(CState* pState);
 
+	void ProcessDirectoryListing(CServer const& server, std::shared_ptr<CDirectoryListing> const& listing, CState const* exempt);
+
 protected:
 	CContextManager();
 
@@ -125,7 +147,7 @@ public:
 	const CServerPath GetRemotePath() const;
 
 	Site const& GetSite() const;
-	CServer const* GetServer() const;
+	ServerWithCredentials const& GetServer() const;
 	wxString GetTitle() const;
 
 	void RefreshLocal();
@@ -185,6 +207,8 @@ public:
 
 	void UpdateSite(wxString const& oldPath, Site const& newSite);
 
+	CStateFilterManager& GetStateFilterManager() { return m_stateFilterManager; }
+
 protected:
 	void SetSite(Site const& site, CServerPath const& path = CServerPath());
 
@@ -194,7 +218,7 @@ protected:
 	std::shared_ptr<CDirectoryListing> m_pDirectoryListing;
 
 	Site m_site;
-	
+
 	wxString m_title;
 	bool m_successful_connect{};
 
@@ -207,12 +231,17 @@ protected:
 	CRemoteRecursiveOperation* m_pRemoteRecursiveOperation;
 
 	CComparisonManager* m_pComparisonManager;
-
-	struct t_handler
+	
+	CStateFilterManager m_stateFilterManager;
+	
+	struct t_handlersForNotification
 	{
-		CStateEventHandler* pHandler;
+		std::vector<CStateEventHandler*> handlers;
+		bool compact_{};
+		bool inNotify_{};
 	};
-	std::vector<t_handler> m_handlers[STATECHANGE_MAX];
+
+	t_handlersForNotification m_handlers[STATECHANGE_MAX];
 
 	CLocalPath GetSynchronizedDirectory(CServerPath remote_path);
 	CServerPath GetSynchronizedDirectory(CLocalPath local_path);
