@@ -1,12 +1,12 @@
-#ifndef __TOOLBAR_H__
-#define __TOOLBAR_H__
+#ifndef FILEZILLA_INTERFACE_TOOLBAR_HEADER
+#define FILEZILLA_INTERFACE_TOOLBAR_HEADER
 
 #include <option_change_event_handler.h>
 #include "state.h"
 
 class CMainFrame;
 
-class CToolBar : public wxToolBar, public CGlobalStateEventHandler, public COptionChangeEventHandler
+class CToolBar final : public wxToolBar, public CGlobalStateEventHandler, public COptionChangeEventHandler
 {
 public:
 	CToolBar() = default;
@@ -38,6 +38,8 @@ protected:
 	std::unique_ptr<wxImageList> toolImages_;
 	std::unique_ptr<wxImageList> disabledToolImages_;
 #endif
+
+	wxSize iconSize_;
 };
 
-#endif //__TOOLBAR_H__
+#endif
