@@ -1,10 +1,12 @@
 #include <filezilla.h>
 #include "buildinfo.h"
 #include "xmlfunctions.h"
+#include "xmlutils.h"
 #include "Options.h"
 #include <wx/ffile.h>
 #include <wx/log.h>
 
+#include <libfilezilla/file.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
 CXmlFile::CXmlFile(std::wstring const& fileName, std::string const& root)
@@ -164,99 +166,6 @@ pugi::xml_node CXmlFile::CreateEmpty()
 	return m_element;
 }
 
-pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, std::string const& value, bool overwrite)
-{
-	return AddTextElementUtf8(node, name, fz::to_utf8(value), overwrite);
-}
-
-pugi::xml_node AddTextElement(pugi::xml_node node, const char* name, std::wstring const& value, bool overwrite)
-{
-	return AddTextElementUtf8(node, name, fz::to_utf8(value), overwrite);
-}
-
-void AddTextElement(pugi::xml_node node, const char* name, int64_t value, bool overwrite)
-{
-	if (overwrite) {
-		node.remove_child(name);
-	}
-	auto child = node.append_child(name);
-	child.text().set(static_cast<long long>(value));
-}
-
-pugi::xml_node AddTextElementUtf8(pugi::xml_node node, const char* name, std::string const& value, bool overwrite)
-{
-	assert(node);
-
-	if (overwrite) {
-		node.remove_child(name);
-	}
-
-	auto element = node.append_child(name);
-	if (!value.empty()) {
-		element.text().set(value.c_str());
-	}
-
-	return element;
-}
-
-void AddTextElement(pugi::xml_node node, std::string const& value)
-{
-	AddTextElementUtf8(node, fz::to_utf8(value));
-}
-
-void AddTextElement(pugi::xml_node node, std::wstring const& value)
-{
-	AddTextElementUtf8(node, fz::to_utf8(value));
-}
-
-void AddTextElement(pugi::xml_node node, int64_t value)
-{
-	assert(node);
-	node.text().set(static_cast<long long>(value));
-}
-
-void AddTextElementUtf8(pugi::xml_node node, std::string const& value)
-{
-	assert(node);
-	node.text().set(value.c_str());
-}
-
-std::wstring GetTextElement_Trimmed(pugi::xml_node node, const char* name)
-{
-	return fz::trimmed(GetTextElement(node, name));
-}
-
-std::wstring GetTextElement(pugi::xml_node node, const char* name)
-{
-	wxASSERT(node);
-
-	return fz::to_wstring_from_utf8(node.child_value(name));
-}
-
-std::wstring GetTextElement_Trimmed(pugi::xml_node node)
-{
-	return fz::trimmed(GetTextElement(node));
-}
-
-std::wstring GetTextElement(pugi::xml_node node)
-{
-	wxASSERT(node);
-
-	return fz::to_wstring_from_utf8(node.child_value());
-}
-
-int64_t GetTextElementInt(pugi::xml_node node, const char* name, int defValue /*=0*/)
-{
-	wxASSERT(node);
-	return static_cast<int64_t>(node.child(name).text().as_llong(defValue));
-}
-
-bool GetTextElementBool(pugi::xml_node node, const char* name, bool defValue /*=false*/)
-{
-	wxASSERT(node);
-	return node.child(name).text().as_bool(defValue);
-}
-
 // Opens the specified XML file if it exists or creates a new one otherwise.
 // Returns false on error.
 bool CXmlFile::GetXmlFile(std::wstring const& file)
@@ -314,8 +223,9 @@ bool CXmlFile::SaveXmlFile()
 	wxString redirectedName = GetRedirectedName();
 	if (fz::local_filesys::get_file_info(fz::to_native(redirectedName), isLink, 0, 0, &flags) == fz::local_filesys::file) {
 #ifdef __WXMSW__
-		if (flags & FILE_ATTRIBUTE_HIDDEN)
+		if (flags & FILE_ATTRIBUTE_HIDDEN) {
 			SetFileAttributes(redirectedName.c_str(), flags & ~FILE_ATTRIBUTE_HIDDEN);
+		}
 #endif
 
 		exists = true;
@@ -330,8 +240,38 @@ bool CXmlFile::SaveXmlFile()
 		}
 	}
 
-	bool success = m_document.save_file(static_cast<wchar_t const*>(redirectedName.c_str()));
+	struct flushing_xml_writer final : public pugi::xml_writer
+	{
+	public:
+		static bool save(pugi::xml_document const& document, std::wstring const& filename)
+		{
+			flushing_xml_writer writer(filename);
+			if (!writer.file_.opened()) {
+				return false;
+			}
+			document.save(writer);
 
+			return writer.file_.opened() && writer.file_.fsync();
+		}
+
+	private:
+		flushing_xml_writer(std::wstring const& filename)
+			: file_(fz::to_native(filename), fz::file::writing, fz::file::empty)
+		{
+		}
+
+		virtual void write(const void* data, size_t size) override {
+			if (file_.opened()) {
+				if (file_.write(data, static_cast<int64_t>(size)) != static_cast<int64_t>(size)) {
+					file_.close();
+				}
+			}
+		}
+
+		fz::file file_;
+	};
+
+	bool success = flushing_xml_writer::save(m_document, redirectedName.ToStdWstring());
 	if (!success) {
 		wxRemoveFile(redirectedName);
 		if (exists) {
@@ -342,13 +282,14 @@ bool CXmlFile::SaveXmlFile()
 		return false;
 	}
 
-	if (exists)
+	if (exists) {
 		wxRemoveFile(redirectedName + _T("~"));
+	}
 
 	return true;
 }
 
-bool GetServer(pugi::xml_node node, CServer& server)
+bool GetServer(pugi::xml_node node, ServerWithCredentials & server)
 {
 	wxASSERT(node);
 
@@ -362,7 +303,7 @@ bool GetServer(pugi::xml_node node, CServer& server)
 		return false;
 	}
 
-	if (!server.SetHost(host, port)) {
+	if (!server.server.SetHost(host, port)) {
 		return false;
 	}
 
@@ -370,105 +311,109 @@ bool GetServer(pugi::xml_node node, CServer& server)
 	if (protocol < 0 || protocol > ServerProtocol::MAX_VALUE) {
 		return false;
 	}
-	server.SetProtocol(static_cast<ServerProtocol>(protocol));
+	server.server.SetProtocol(static_cast<ServerProtocol>(protocol));
 
 	int type = GetTextElementInt(node, "Type");
 	if (type < 0 || type >= SERVERTYPE_MAX) {
 		return false;
 	}
 
-	server.SetType(static_cast<ServerType>(type));
+	server.server.SetType(static_cast<ServerType>(type));
 
 	int logonType = GetTextElementInt(node, "Logontype");
-	if (logonType < 0 || logonType >= LOGONTYPE_MAX) {
+	if (logonType < 0 || logonType >= static_cast<int>(LogonType::count)) {
 		return false;
 	}
 
 	server.SetLogonType(static_cast<LogonType>(logonType));
-
-	if (server.GetLogonType() != ANONYMOUS) {
+	
+	if (server.credentials.logonType_ != LogonType::anonymous) {
 		std::wstring user = GetTextElement(node, "User");
+		if (user.empty() && server.credentials.logonType_ != LogonType::interactive && server.credentials.logonType_ != LogonType::ask) {
+			return false;
+		}
 
 		std::wstring pass, key;
-		if ((long)NORMAL == logonType || (long)ACCOUNT == logonType) {
+		if (server.credentials.logonType_ == LogonType::normal || server.credentials.logonType_ == LogonType::account) {
 			auto passElement = node.child("Pass");
 			if (passElement) {
-
 				std::wstring encoding = GetTextAttribute(passElement, "encoding");
 
 				if (encoding == _T("base64")) {
 					std::string decoded = fz::base64_decode(passElement.child_value());
 					pass = fz::to_wstring_from_utf8(decoded);
 				}
+				else if (encoding == _T("crypt")) {
+					pass = fz::to_wstring_from_utf8(passElement.child_value());
+					server.credentials.encrypted_ = public_key::from_base64(passElement.attribute("pubkey").value());
+					if (!server.credentials.encrypted_) {
+						pass.clear();
+						server.SetLogonType(LogonType::ask);
+					}
+				}
 				else if (!encoding.empty()) {
-					server.SetLogonType(ASK);
+					server.SetLogonType(LogonType::ask);
 				}
 				else {
 					pass = GetTextElement(passElement);
 				}
 			}
 		}
-		else if ((long)KEY == logonType) {
+		else if (server.credentials.logonType_ == LogonType::key) {
 			key = GetTextElement(node, "Keyfile");
 
 			// password should be empty if we're using a key file
 			pass.clear();
 
-			server.SetKeyFile(key);
+			server.credentials.keyFile_ = key;
 		}
 
-		if (!server.SetUser(user, pass)) {
-			return false;
-		}
+		server.SetUser(user);
+		server.credentials.SetPass(pass);
 
-		if ((long)ACCOUNT == logonType) {
-			std::wstring account = GetTextElement(node, "Account");
-			if (account.empty()) {
-				return false;
-			}
-			if (!server.SetAccount(account)) {
-				return false;
-			}
-		}
+		server.credentials.account_ = GetTextElement(node, "Account");
 	}
 
 	int timezoneOffset = GetTextElementInt(node, "TimezoneOffset");
-	if (!server.SetTimezoneOffset(timezoneOffset)) {
+	if (!server.server.SetTimezoneOffset(timezoneOffset)) {
 		return false;
 	}
 
 	wxString pasvMode = GetTextElement(node, "PasvMode");
-	if (pasvMode == _T("MODE_PASSIVE"))
-		server.SetPasvMode(MODE_PASSIVE);
-	else if (pasvMode == _T("MODE_ACTIVE"))
-		server.SetPasvMode(MODE_ACTIVE);
-	else
-		server.SetPasvMode(MODE_DEFAULT);
+	if (pasvMode == _T("MODE_PASSIVE")) {
+		server.server.SetPasvMode(MODE_PASSIVE);
+	}
+	else if (pasvMode == _T("MODE_ACTIVE")) {
+		server.server.SetPasvMode(MODE_ACTIVE);
+	}
+	else {
+		server.server.SetPasvMode(MODE_DEFAULT);
+	}
 
 	int maximumMultipleConnections = GetTextElementInt(node, "MaximumMultipleConnections");
-	server.MaximumMultipleConnections(maximumMultipleConnections);
+	server.server.MaximumMultipleConnections(maximumMultipleConnections);
 
 	wxString encodingType = GetTextElement(node, "EncodingType");
 	if (encodingType == _T("Auto")) {
-		server.SetEncodingType(ENCODING_AUTO);
+		server.server.SetEncodingType(ENCODING_AUTO);
 	}
 	else if (encodingType == _T("UTF-8")) {
-		server.SetEncodingType(ENCODING_UTF8);
+		server.server.SetEncodingType(ENCODING_UTF8);
 	}
 	else if (encodingType == _T("Custom")) {
 		std::wstring customEncoding = GetTextElement(node, "CustomEncoding");
 		if (customEncoding.empty()) {
 			return false;
 		}
-		if (!server.SetEncodingType(ENCODING_CUSTOM, customEncoding)) {
+		if (!server.server.SetEncodingType(ENCODING_CUSTOM, customEncoding)) {
 			return false;
 		}
 	}
 	else {
-		server.SetEncodingType(ENCODING_AUTO);
+		server.server.SetEncodingType(ENCODING_AUTO);
 	}
 
-	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
+	if (CServer::SupportsPostLoginCommands(server.server.GetProtocol())) {
 		std::vector<std::wstring> postLoginCommands;
 		auto element = node.child("PostLoginCommands");
 		if (element) {
@@ -479,68 +424,72 @@ bool GetServer(pugi::xml_node node, CServer& server)
 				}
 			}
 		}
-		if (!server.SetPostLoginCommands(postLoginCommands)) {
+		if (!server.server.SetPostLoginCommands(postLoginCommands)) {
 			return false;
 		}
 	}
 
-	server.SetBypassProxy(GetTextElementInt(node, "BypassProxy", false) == 1);
-	server.SetName(GetTextElement_Trimmed(node, "Name"));
+	server.server.SetBypassProxy(GetTextElementInt(node, "BypassProxy", false) == 1);
+	server.server.SetName(GetTextElement_Trimmed(node, "Name"));
 
-	if (server.GetName().empty()) {
-		server.SetName(GetTextElement_Trimmed(node));
+	if (server.server.GetName().empty()) {
+		server.server.SetName(GetTextElement_Trimmed(node));
 	}
 
 	return true;
 }
 
-void SetServer(pugi::xml_node node, const CServer& server)
+void SetServer(pugi::xml_node node, ServerWithCredentials const& server)
 {
 	if (!node) {
 		return;
 	}
 
-	bool kiosk_mode = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0;
-
 	for (auto child = node.first_child(); child; child = node.first_child()) {
 		node.remove_child(child);
 	}
 
-	AddTextElement(node, "Host", server.GetHost());
-	AddTextElement(node, "Port", server.GetPort());
-	AddTextElement(node, "Protocol", server.GetProtocol());
-	AddTextElement(node, "Type", server.GetType());
+	AddTextElement(node, "Host", server.server.GetHost());
+	AddTextElement(node, "Port", server.server.GetPort());
+	AddTextElement(node, "Protocol", server.server.GetProtocol());
+	AddTextElement(node, "Type", server.server.GetType());
 
-	LogonType logonType = server.GetLogonType();
+	ProtectedCredentials credentials = server.credentials;
 
-	if (server.GetLogonType() != ANONYMOUS) {
-		AddTextElement(node, "User", server.GetUser());
+	if (credentials.logonType_ != LogonType::anonymous) {
+		AddTextElement(node, "User", server.server.GetUser());
 
-		if (server.GetLogonType() == NORMAL || server.GetLogonType() == ACCOUNT) {
-			if (kiosk_mode) {
-				logonType = ASK;
+		credentials.Protect();
+
+		if (credentials.logonType_ == LogonType::normal || credentials.logonType_ == LogonType::account) {
+			std::string pass = fz::to_utf8(credentials.GetPass());
+
+			if (credentials.encrypted_) {
+				pugi::xml_node passElement = AddTextElementUtf8(node, "Pass", pass);
+				if (passElement) {
+					SetTextAttribute(passElement, "encoding", _T("crypt"));
+					SetTextAttributeUtf8(passElement, "pubkey", credentials.encrypted_.to_base64());
+				}
 			}
 			else {
-				std::string pass = fz::to_utf8(server.GetPass());
-				
 				pugi::xml_node passElement = AddTextElementUtf8(node, "Pass", fz::base64_encode(pass));
 				if (passElement) {
 					SetTextAttribute(passElement, "encoding", _T("base64"));
 				}
+			}
 
-				if (server.GetLogonType() == ACCOUNT) {
-					AddTextElement(node, "Account", server.GetAccount());
-				}
+			if (credentials.logonType_ == LogonType::account) {
+				AddTextElement(node, "Account", credentials.account_);
 			}
 		}
-		else if (server.GetLogonType() == KEY) {
-			AddTextElement(node, "Keyfile", server.GetKeyFile());
+		else if (!credentials.keyFile_.empty()) {
+			AddTextElement(node, "Keyfile", credentials.keyFile_);
 		}
 	}
-	AddTextElement(node, "Logontype", logonType);
+	AddTextElement(node, "Logontype", static_cast<int>(credentials.logonType_));
 
-	AddTextElement(node, "TimezoneOffset", server.GetTimezoneOffset());
-	switch (server.GetPasvMode())
+	AddTextElement(node, "TimezoneOffset", server.server.GetTimezoneOffset());
+	switch (server.server.GetPasvMode())
 	{
 	case MODE_PASSIVE:
 		AddTextElementUtf8(node, "PasvMode", "MODE_PASSIVE");
@@ -552,9 +501,9 @@ void SetServer(pugi::xml_node node, const CServer& server)
 		AddTextElementUtf8(node, "PasvMode", "MODE_DEFAULT");
 		break;
 	}
-	AddTextElement(node, "MaximumMultipleConnections", server.MaximumMultipleConnections());
+	AddTextElement(node, "MaximumMultipleConnections", server.server.MaximumMultipleConnections());
 
-	switch (server.GetEncodingType())
+	switch (server.server.GetEncodingType())
 	{
 	case ENCODING_AUTO:
 		AddTextElementUtf8(node, "EncodingType", "Auto");
@@ -564,84 +513,27 @@ void SetServer(pugi::xml_node node, const CServer& server)
 		break;
 	case ENCODING_CUSTOM:
 		AddTextElementUtf8(node, "EncodingType", "Custom");
-		AddTextElement(node, "CustomEncoding", server.GetCustomEncoding());
+		AddTextElement(node, "CustomEncoding", server.server.GetCustomEncoding());
 		break;
 	}
 
-	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
-		std::vector<std::wstring> const& postLoginCommands = server.GetPostLoginCommands();
+	if (CServer::SupportsPostLoginCommands(server.server.GetProtocol())) {
+		std::vector<std::wstring> const& postLoginCommands = server.server.GetPostLoginCommands();
 		if (!postLoginCommands.empty()) {
 			auto element = node.append_child("PostLoginCommands");
 			for (auto const& command : postLoginCommands) {
 				AddTextElement(element, "Command", command);
-			}				
+			}
 		}
 	}
 
-	AddTextElementUtf8(node, "BypassProxy", server.GetBypassProxy() ? "1" : "0");
-	std::wstring const& name = server.GetName();
+	AddTextElementUtf8(node, "BypassProxy", server.server.GetBypassProxy() ? "1" : "0");
+	std::wstring const& name = server.server.GetName();
 	if (!name.empty()) {
 		AddTextElement(node, "Name", name);
 	}
 }
 
-void SetTextAttribute(pugi::xml_node node, char const* name, std::string const& value)
-{
-	SetTextAttributeUtf8(node, name, fz::to_utf8(value));
-}
-
-void SetTextAttribute(pugi::xml_node node, char const* name, std::wstring const& value)
-{
-	SetTextAttributeUtf8(node, name, fz::to_utf8(value));
-}
-
-void SetTextAttributeUtf8(pugi::xml_node node, char const* name, std::string const& utf8)
-{
-	assert(node);
-	auto attribute = node.attribute(name);
-	if (!attribute) {
-		attribute = node.append_attribute(name);
-	}
-	attribute.set_value(utf8.c_str());
-}
-
-std::wstring GetTextAttribute(pugi::xml_node node, char const* name)
-{
-	assert(node);
-
-	const char* value = node.attribute(name).value();
-	return fz::to_wstring_from_utf8(value);
-}
-
-pugi::xml_node FindElementWithAttribute(pugi::xml_node node, const char* element, const char* attribute, const char* value)
-{
-	pugi::xml_node child = element ? node.child(element) : node.first_child();
-	while (child) {
-		const char* nodeVal = child.attribute(attribute).value();
-		if (nodeVal && !strcmp(value, nodeVal)) {
-			return child;
-		}
-
-		child = element ? child.next_sibling(element) : child.next_sibling();
-	}
-
-	return child;
-}
-
-int GetAttributeInt(pugi::xml_node node, const char* name)
-{
-	return node.attribute(name).as_int();
-}
-
-void SetAttributeInt(pugi::xml_node node, const char* name, int value)
-{
-	auto attribute = node.attribute(name);
-	if (!attribute) {
-		attribute = node.append_attribute(name);
-	}
-	attribute.set_value(value);
-}
-
 namespace {
 struct xml_memory_writer : pugi::xml_writer
 {
@@ -663,8 +555,9 @@ struct xml_memory_writer : pugi::xml_writer
 
 size_t CXmlFile::GetRawDataLength()
 {
-	if (!m_document)
+	if (!m_document) {
 		return 0;
+	}
 
 	xml_memory_writer writer;
 	m_document.save(writer);
